@@ -44,10 +44,12 @@
 # warning Uho, your IPv6 support is broken and has been disabled. Fix your C library.
 # undef AF_INET6
 #endif
-
 #ifndef AF_LOCAL
 # define AF_LOCAL AF_UNIX
 #endif
+#if !defined(MSG_NOSIGNAL) && defined(SO_NOSIGPIPE)
+# define MSG_NOSIGNAL 0
+#endif
 /* Required yet non-standard cmsg functions */
 #ifndef CMSG_ALIGN
 # define CMSG_ALIGN(len) (((len) + sizeof(intptr_t)-1) & ~(sizeof(intptr_t)-1))
@@ -68,7 +70,7 @@ static inline int is_allowed_port (uint16_t port)
 
 static inline int send_err (int fd, int err)
 {
-    return send (fd, &err, sizeof (err), 0) == sizeof (err) ? 0 : -1;
+    return send(fd, &err, sizeof (err), MSG_NOSIGNAL) == sizeof (err) ? 0 : -1;
 }
 
 /**
@@ -99,7 +101,7 @@ static int send_fd (int p, int fd)
     memcpy (CMSG_DATA (cmsg), &fd, sizeof (fd));
     hdr.msg_controllen = cmsg->cmsg_len;
 
-    return sendmsg (p, &hdr, 0) == sizeof (val) ? 0 : -1;
+    return sendmsg(p, &hdr, MSG_NOSIGNAL) == sizeof (val) ? 0 : -1;
 }
 
 
@@ -193,6 +195,10 @@ int main (int argc, char *argv[])
     if (pair[0] < 3)
         goto error; /* we want 0, 1 and 2 open */
 
+#ifdef SO_NOSIGPIPE
+    setsockopt(pair[1], SOL_SOCKET, SO_NOSIGPIPE, &(int){ 1 }, sizeof (int));
+#endif
+
     pid_t pid = fork ();
     switch (pid)
     {
@@ -239,7 +245,7 @@ int main (int argc, char *argv[])
     }
     if (uid == 0)
     {
-        fprintf (stderr, "Cannot determine unprivileged user for VLC!\n");
+        fputs("Cannot determine unprivileged user for VLC!\n", stderr);
         exit (1);
     }
     setuid (uid);
