@@ -2,7 +2,7 @@
  * media_list.c: libvlc new API media list functions
  *****************************************************************************
  * Copyright (C) 2007 VLC authors and VideoLAN
- * $Id: 82fee60fdb24cdb9f4bc4ba063db6deeaa66e750 $
+ * $Id: 7c93e41d5f67094e240c28d49633d455764f9970 $
  *
  * Authors: Pierre d'Herbemont <pdherbemont # videolan.org>
  *
@@ -78,7 +78,7 @@ notify_item_addition( libvlc_media_list_t * p_mlist,
     }
 
     /* Send the event */
-    libvlc_event_send( p_mlist->p_event_manager, &event );
+    libvlc_event_send( &p_mlist->event_manager, &event );
 }
 
 /**************************************************************************
@@ -109,7 +109,7 @@ notify_item_deletion( libvlc_media_list_t * p_mlist,
     }
 
     /* Send the event */
-    libvlc_event_send( p_mlist->p_event_manager, &event );
+    libvlc_event_send( &p_mlist->event_manager, &event );
 }
 
 /* LibVLC internal */
@@ -120,7 +120,7 @@ void libvlc_media_list_internal_end_reached( libvlc_media_list_t * p_mlist )
     event.type = libvlc_MediaListEndReached;
 
     /* Send the event */
-    libvlc_event_send( p_mlist->p_event_manager, &event );
+    libvlc_event_send( &p_mlist->event_manager, &event );
 }
 
 /**************************************************************************
@@ -160,13 +160,7 @@ libvlc_media_list_new( libvlc_instance_t * p_inst )
     }
 
     p_mlist->p_libvlc_instance = p_inst;
-    p_mlist->p_event_manager = libvlc_event_manager_new( p_mlist );
-    if( unlikely(p_mlist->p_event_manager == NULL) )
-    {
-        free(p_mlist);
-        return NULL;
-    }
-
+    libvlc_event_manager_init( &p_mlist->event_manager, p_mlist );
     p_mlist->b_read_only = false;
 
     vlc_mutex_init( &p_mlist->object_lock );
@@ -189,9 +183,6 @@ libvlc_media_list_new( libvlc_instance_t * p_inst )
  **************************************************************************/
 void libvlc_media_list_release( libvlc_media_list_t * p_mlist )
 {
-    libvlc_media_t * p_md;
-    int i;
-
     vlc_mutex_lock( &p_mlist->refcount_lock );
     p_mlist->i_refcount--;
     if( p_mlist->i_refcount > 0 )
@@ -203,13 +194,12 @@ void libvlc_media_list_release( libvlc_media_list_t * p_mlist )
 
     /* Refcount null, time to free */
 
-    libvlc_event_manager_release( p_mlist->p_event_manager );
-
+    libvlc_event_manager_destroy( &p_mlist->event_manager );
     libvlc_media_release( p_mlist->p_md );
 
-    for ( i = 0; i < vlc_array_count( &p_mlist->items ); i++ )
+    for( size_t i = 0; i < vlc_array_count( &p_mlist->items ); i++ )
     {
-        p_md = vlc_array_item_at_index( &p_mlist->items, i );
+        libvlc_media_t* p_md = vlc_array_item_at_index( &p_mlist->items, i );
         libvlc_media_release( p_md );
     }
 
@@ -256,7 +246,7 @@ libvlc_media_list_add_file_content( libvlc_media_list_t * p_mlist,
                                              p_input_item );
     if( !p_md )
     {
-        vlc_gc_decref( p_input_item );
+        input_item_Release( p_input_item );
         return -1;
     }
 
@@ -345,7 +335,7 @@ void libvlc_media_list_internal_add_media( libvlc_media_list_t * p_mlist,
 
     notify_item_addition( p_mlist, p_md, vlc_array_count( &p_mlist->items ),
                           EventWillHappen );
-    vlc_array_append( &p_mlist->items, p_md );
+    vlc_array_append_or_abort( &p_mlist->items, p_md );
     notify_item_addition( p_mlist, p_md, vlc_array_count( &p_mlist->items )-1,
                           EventDidHappen );
 }
@@ -373,7 +363,7 @@ void libvlc_media_list_internal_insert_media( libvlc_media_list_t * p_mlist,
     libvlc_media_retain( p_md );
 
     notify_item_addition( p_mlist, p_md, index, EventWillHappen );
-    vlc_array_insert( &p_mlist->items, p_md, index );
+    vlc_array_insert_or_abort( &p_mlist->items, p_md, index );
     notify_item_addition( p_mlist, p_md, index, EventDidHappen );
 }
 
@@ -396,7 +386,7 @@ int libvlc_media_list_internal_remove_index( libvlc_media_list_t * p_mlist,
 {
     libvlc_media_t * p_md;
 
-    if( index < 0 || index >= vlc_array_count( &p_mlist->items ))
+    if( (size_t) index >= vlc_array_count( &p_mlist->items ))
     {
         libvlc_printerr( "Index out of bounds" );
         return -1;
@@ -423,7 +413,7 @@ libvlc_media_list_item_at_index( libvlc_media_list_t * p_mlist,
 {
     libvlc_media_t * p_md;
 
-    if( index < 0 || index >= vlc_array_count( &p_mlist->items ))
+    if( (size_t) index >= vlc_array_count( &p_mlist->items ))
     {
         libvlc_printerr( "Index out of bounds" );
         return NULL;
@@ -443,16 +433,11 @@ libvlc_media_list_item_at_index( libvlc_media_list_t * p_mlist,
 int libvlc_media_list_index_of_item( libvlc_media_list_t * p_mlist,
                                      libvlc_media_t * p_searched_md )
 {
-    libvlc_media_t * p_md;
-    int i;
-    for ( i = 0; i < vlc_array_count( &p_mlist->items ); i++ )
-    {
-        p_md = vlc_array_item_at_index( &p_mlist->items, i );
-        if( p_searched_md == p_md )
-            return i;
-    }
-    libvlc_printerr( "Media not found" );
-    return -1;
+    int idx = vlc_array_index_of_item( &p_mlist->items, p_searched_md );
+    if( idx == -1 )
+        libvlc_printerr( "Media not found" );
+
+    return idx;
 }
 
 /**************************************************************************
@@ -489,12 +474,12 @@ void libvlc_media_list_unlock( libvlc_media_list_t * p_mlist )
 
 
 /**************************************************************************
- *       libvlc_media_list_p_event_manager (Public)
+ *       libvlc_media_list_event_manager (Public)
  *
  * The p_event_manager is immutable, so you don't have to hold the lock
  **************************************************************************/
 libvlc_event_manager_t *
 libvlc_media_list_event_manager( libvlc_media_list_t * p_mlist )
 {
-    return p_mlist->p_event_manager;
+    return &p_mlist->event_manager;
 }
