@@ -2,7 +2,7 @@
  * crossbar.cpp : DirectShow access module for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2004, 2009 VLC authors and VideoLAN
- * $Id: 7b163f6088376d6fd03d81b937ca0b8451c5988e $
+ * $Id: 75e93182b342ceae5153c83a08abea4139f0796e $
  *
  * Author: Damien Fouilleul <damien dot fouilleul at laposte dot net>
  *
@@ -80,7 +80,7 @@ void DeleteCrossbarRoutes( access_sys_t *p_sys )
     /* Remove crossbar filters from graph */
     for( int i = 0; i < p_sys->i_crossbar_route_depth; i++ )
     {
-        p_sys->crossbar_routes[i].pXbar->Release();
+        p_sys->crossbar_routes[i].pXbar.Reset();
     }
     p_sys->i_crossbar_route_depth = 0;
 }
@@ -89,30 +89,27 @@ void DeleteCrossbarRoutes( access_sys_t *p_sys )
  * RouteCrossbars (Does not AddRef the returned *Pin)
  *****************************************************************************/
 static HRESULT GetCrossbarIPinAtIndex( IAMCrossbar *pXbar, LONG PinIndex,
-                                       BOOL IsInputPin, IPin ** ppPin )
+                                       BOOL IsInputPin, ComPtr<IPin>* ppPin )
 {
     LONG         cntInPins, cntOutPins;
-    IPin        *pP = NULL;
-    IBaseFilter *pFilter = NULL;
-    IEnumPins   *pins = NULL;
+    ComPtr<IPin> pP;
+    ComPtr<IBaseFilter> pFilter;
+    ComPtr<IEnumPins> pins;
     ULONG        n;
 
     if( !pXbar || !ppPin ) return E_POINTER;
 
-    *ppPin = 0;
-
     if( S_OK != pXbar->get_PinCounts(&cntOutPins, &cntInPins) ) return E_FAIL;
 
     LONG TrueIndex = IsInputPin ? PinIndex : PinIndex + cntInPins;
 
-    if( pXbar->QueryInterface(IID_IBaseFilter, (void **)&pFilter) == S_OK )
+    if( pXbar->QueryInterface(IID_IBaseFilter, (void **)pFilter.GetAddressOf()) == S_OK )
     {
-        if( SUCCEEDED(pFilter->EnumPins(&pins)) )
+        if( SUCCEEDED(pFilter->EnumPins(pins.GetAddressOf())) )
         {
             LONG i = 0;
-            while( pins->Next(1, &pP, &n) == S_OK )
+            while( pins->Next(1, pP.ReleaseAndGetAddressOf(), &n) == S_OK )
             {
-                pP->Release();
                 if( i == TrueIndex )
                 {
                     *ppPin = pP;
@@ -120,9 +117,7 @@ static HRESULT GetCrossbarIPinAtIndex( IAMCrossbar *pXbar, LONG PinIndex,
                 }
                 i++;
             }
-            pins->Release();
         }
-        pFilter->Release();
     }
 
     return *ppPin ? S_OK : E_FAIL;
@@ -135,9 +130,9 @@ static HRESULT GetCrossbarIndexFromIPin( IAMCrossbar * pXbar, LONG * PinIndex,
                                          BOOL IsInputPin, IPin * pPin )
 {
     LONG         cntInPins, cntOutPins;
-    IPin        *pP = NULL;
-    IBaseFilter *pFilter = NULL;
-    IEnumPins   *pins = NULL;
+    ComPtr<IPin> pP;
+    ComPtr<IBaseFilter> pFilter;
+    ComPtr<IEnumPins>   pins;
     ULONG        n;
     BOOL         fOK = FALSE;
 
@@ -147,16 +142,15 @@ static HRESULT GetCrossbarIndexFromIPin( IAMCrossbar * pXbar, LONG * PinIndex,
     if( S_OK != pXbar->get_PinCounts(&cntOutPins, &cntInPins) )
         return E_FAIL;
 
-    if( pXbar->QueryInterface(IID_IBaseFilter, (void **)&pFilter) == S_OK )
+    if( pXbar->QueryInterface(IID_IBaseFilter, (void **)pFilter.GetAddressOf()) == S_OK )
     {
         if( SUCCEEDED(pFilter->EnumPins(&pins)) )
         {
             LONG i=0;
 
-            while( pins->Next(1, &pP, &n) == S_OK )
+            while( pins->Next(1, pP.ReleaseAndGetAddressOf(), &n) == S_OK )
             {
-                pP->Release();
-                if( pPin == pP )
+                if( pPin == pP.Get() )
                 {
                     *PinIndex = IsInputPin ? i : i - cntInPins;
                     fOK = TRUE;
@@ -164,9 +158,7 @@ static HRESULT GetCrossbarIndexFromIPin( IAMCrossbar * pXbar, LONG * PinIndex,
                 }
                 i++;
             }
-            pins->Release();
         }
-        pFilter->Release();
     }
 
     return fOK ? S_OK : E_FAIL;
@@ -180,48 +172,42 @@ HRESULT FindCrossbarRoutes( vlc_object_t *p_this, access_sys_t *p_sys,
 {
     HRESULT result = S_FALSE;
 
-    IPin *p_output_pin;
-    if( FAILED(p_input_pin->ConnectedTo(&p_output_pin)) ) return S_FALSE;
+    ComPtr<IPin> p_output_pin;
+    if( FAILED(p_input_pin->ConnectedTo(p_output_pin.GetAddressOf())) ) return S_FALSE;
 
     // It is connected, so now find out if the filter supports IAMCrossbar
     PIN_INFO pinInfo;
     if( FAILED(p_output_pin->QueryPinInfo(&pinInfo)) ||
         PINDIR_OUTPUT != pinInfo.dir )
     {
-        p_output_pin->Release ();
         return S_FALSE;
     }
 
-    IAMCrossbar *pXbar = NULL;
+    ComPtr<IAMCrossbar> pXbar;
     if( FAILED(pinInfo.pFilter->QueryInterface(IID_IAMCrossbar,
-                                               (void **)&pXbar)) )
+                                               (void **)pXbar.GetAddressOf())) )
     {
         pinInfo.pFilter->Release();
-        p_output_pin->Release ();
         return S_FALSE;
     }
 
     LONG inputPinCount, outputPinCount;
     if( FAILED(pXbar->get_PinCounts(&outputPinCount, &inputPinCount)) )
     {
-        pXbar->Release();
         pinInfo.pFilter->Release();
-        p_output_pin->Release ();
         return S_FALSE;
     }
 
     LONG inputPinIndexRelated, outputPinIndexRelated;
     LONG inputPinPhysicalType = 0, outputPinPhysicalType;
     LONG inputPinIndex = 0, outputPinIndex;
-    if( FAILED(GetCrossbarIndexFromIPin( pXbar, &outputPinIndex,
-                                         FALSE, p_output_pin )) ||
+    if( FAILED(GetCrossbarIndexFromIPin( pXbar.Get(), &outputPinIndex,
+                                         FALSE, p_output_pin.Get() )) ||
         FAILED(pXbar->get_CrossbarPinInfo( FALSE, outputPinIndex,
                                            &outputPinIndexRelated,
                                            &outputPinPhysicalType )) )
     {
-        pXbar->Release();
         pinInfo.pFilter->Release();
-        p_output_pin->Release ();
         return S_FALSE;
     }
 
@@ -240,11 +226,11 @@ HRESULT FindCrossbarRoutes( vlc_object_t *p_this, access_sys_t *p_sys,
             {
                 // remember connector type
                 physicalType = inputPinPhysicalType;
- 
+
                 msg_Dbg( p_this, "found existing route for output %ld (type %s) to input %ld (type %s)",
                          outputPinIndex, GetPhysicalPinName( outputPinPhysicalType ),
                          inputPinIndex, GetPhysicalPinName( inputPinPhysicalType ) );
- 
+
                 // fall through to for loop, note 'inputPinIndex' is set to the pin we are looking for
                 // hence, loop iteration should not wind back
 
@@ -255,7 +241,7 @@ HRESULT FindCrossbarRoutes( vlc_object_t *p_this, access_sys_t *p_sys,
             inputPinIndex = 0;
         }
     }
- 
+
     //
     // for all input pins
     //
@@ -269,22 +255,19 @@ HRESULT FindCrossbarRoutes( vlc_object_t *p_this, access_sys_t *p_sys,
 
         // Can we route it?
         if( FAILED(pXbar->CanRoute(outputPinIndex, inputPinIndex)) ) continue;
- 
- 
-        IPin *pPin;
-        if( FAILED(GetCrossbarIPinAtIndex( pXbar, inputPinIndex,
-                                           TRUE, &pPin)) ) continue;
 
-        result = FindCrossbarRoutes( p_this, p_sys, pPin,
+
+        ComPtr<IPin> pPin;
+        if( FAILED(GetCrossbarIPinAtIndex( pXbar.Get(), inputPinIndex,
+                                           TRUE, &pPin ) ) ) continue;
+
+        result = FindCrossbarRoutes( p_this, p_sys, pPin.Get(),
                                      physicalType, depth+1 );
 
         if( S_OK == result || (S_FALSE == result &&
             physicalType == inputPinPhysicalType &&
             (p_sys->i_crossbar_route_depth = depth+1) < MAX_CROSSBAR_DEPTH) )
         {
-            // hold on crossbar, will be released when graph is destroyed
-            pXbar->AddRef();
-
             // remember crossbar route
             p_sys->crossbar_routes[depth].pXbar = pXbar;
             p_sys->crossbar_routes[depth].VideoInputIndex = inputPinIndex;
@@ -300,10 +283,7 @@ HRESULT FindCrossbarRoutes( vlc_object_t *p_this, access_sys_t *p_sys,
             result = S_OK;
         }
     }
-
-    pXbar->Release();
     pinInfo.pFilter->Release();
-    p_output_pin->Release ();
 
     return result;
 }
