@@ -123,6 +123,7 @@ int rtp_add_type (demux_t *demux, rtp_session_t *ses, const rtp_pt_t *pt)
     ppt->init = pt->init ? pt->init : no_init;
     ppt->destroy = pt->destroy ? pt->destroy : no_destroy;
     ppt->decode = pt->decode ? pt->decode : no_decode;
+    ppt->header = NULL;
     ppt->frequency = pt->frequency;
     ppt->number = pt->number;
     msg_Dbg (demux, "added payload type %"PRIu8" (f = %"PRIu32" Hz)",
@@ -355,7 +356,7 @@ rtp_queue (demux_t *demux, rtp_session_t *session, block_t *block)
     block_t **pp = &src->blocks;
     for (block_t *prev = *pp; prev != NULL; prev = *pp)
     {
-        int16_t delta_seq = seq - rtp_seq (prev);
+        delta_seq = seq - rtp_seq (prev);
         if (delta_seq < 0)
             break;
         if (delta_seq == 0)
@@ -514,6 +515,9 @@ rtp_decode (demux_t *demux, const rtp_session_t *session, rtp_source_t *src)
         goto drop;
     }
 
+    if(pt->header)
+        pt->header(demux, pt_data, block);
+
     /* Computes the PTS from the RTP timestamp and payload RTP frequency.
      * DTS is unknown. Also, while the clock frequency depends on the payload
      * format, a single source MUST only use payloads of a chosen frequency.
