@@ -25,6 +25,7 @@
 #endif
 #include <stdarg.h>
 #include <assert.h>
+#include <errno.h>
 #include <xcb/xcb.h>
 #include <xcb/composite.h>
 #ifdef HAVE_SYS_SHM_H
@@ -169,7 +170,7 @@ static int Open (vlc_object_t *obj)
         {
             if (snum == 0)
             {
-               scr = i.data;
+                scr = i.data;
                 break;
             }
             snum--;
@@ -348,8 +349,8 @@ discard:
         return;
     }
 
-    int w = sys->w;
-    int h = sys->h;
+    unsigned w = sys->w;
+    unsigned h = sys->h;
     int x, y;
 
     if (sys->follow_mouse)
@@ -365,9 +366,9 @@ discard:
         if (w == 0 || w > geo->width)
             w = geo->width;
         x = ptr->win_x;
-        if (x < w / 2)
+        if (x < (int)(w / 2))
             x = 0;
-        else if (x >= (int)geo->width - (w / 2))
+        else if (x >= (int)(geo->width - (w / 2)))
             x = geo->width - w;
         else
             x -= w / 2;
@@ -375,9 +376,9 @@ discard:
         if (h == 0 || h > geo->height)
             h = geo->height;
         y = ptr->win_y;
-        if (y < h / 2)
+        if (y < (int)(h / 2))
             y = 0;
-        else if (y >= (int)geo->height - (h / 2))
+        else if (y >= (int)(geo->height - (h / 2)))
             y = geo->height - h;
         else
             y -= h / 2;
@@ -390,14 +391,14 @@ discard:
         max = (int)geo->width - x;
         if (max <= 0)
             goto discard;
-        if (w == 0 || w > max)
+        if (w == 0 || w > (unsigned)max)
             w = max;
 
         y = sys->y;
         max = (int)geo->height - y;
         if (max <= 0)
             goto discard;
-        if (h == 0 || h > max)
+        if (h == 0 || h > (unsigned)max)
             h = max;
     }
 
@@ -438,7 +439,8 @@ discard:
         int id = shmget (IPC_PRIVATE, size, IPC_CREAT | 0777);
         if (id == -1) /* XXX: fallback */
         {
-            msg_Err (demux, "shared memory allocation error: %m");
+            msg_Err (demux, "shared memory allocation error: %s",
+                     vlc_strerror_c(errno));
             goto noshm;
         }
 
@@ -465,7 +467,8 @@ discard:
         shmctl (id, IPC_RMID, 0);
         if (-1 == (intptr_t)shm)
         {
-            msg_Err (demux, "shared memory attachment error: %m");
+            msg_Err (demux, "shared memory attachment error: %s",
+                     vlc_strerror_c(errno));
             return;
         }
 
@@ -499,9 +502,11 @@ noshm:
     {
         block->i_pts = block->i_dts = mdate ();
 
-        es_out_Control (demux->out, ES_OUT_SET_PCR, block->i_pts);
+        es_out_SetPCR(demux->out, block->i_pts);
         es_out_Send (demux->out, sys->es, block);
     }
+    else
+        block_Release (block);
 }
 
 static es_out_id_t *InitES (demux_t *demux, uint_fast16_t width,
@@ -522,7 +527,7 @@ static es_out_id_t *InitES (demux_t *demux, uint_fast16_t width,
         {
             case 32:
                 if (fmt->bits_per_pixel == 32)
-                    chroma = VLC_CODEC_RGBA;
+                    chroma = VLC_CODEC_ARGB;
                 break;
             case 24:
                 if (fmt->bits_per_pixel == 32)
