@@ -423,19 +423,16 @@ static int D3dCreateDevice(vlc_va_t *va)
         return VLC_SUCCESS;
     }
 
-#if VLC_WINSTORE_APP
-    sys->d3d_dev.d3dcontext = var_InheritInteger(va, "winrt-d3dcontext");
-    if (likely(sys->d3d_dev.d3dcontext))
+    sys->d3d_dev.d3dcontext = (void*)var_InheritInteger(va, "winrt-d3dcontext");
+    if (unlikely(sys->d3d_dev.d3dcontext))
     {
-        ID3D11Device* d3ddevice = NULL;
         ID3D11DeviceContext_GetDevice(sys->d3d_dev.d3dcontext, &sys->d3d_dev.d3ddevice);
         ID3D11DeviceContext_AddRef(sys->d3d_dev.d3dcontext);
         ID3D11Device_Release(sys->d3d_dev.d3ddevice);
     }
-#endif
 
     /* */
-    if (!sys->d3d_dev.d3ddevice)
+    if (likely(!sys->d3d_dev.d3ddevice))
     {
         hr = D3D11_CreateDevice(va, &sys->hd3d, true, &sys->d3d_dev);
         if (FAILED(hr)) {
