@@ -2,7 +2,7 @@
  * SSA/ASS subtitle decoder using libass.
  *****************************************************************************
  * Copyright (C) 2008-2009 VLC authors and VideoLAN
- * $Id: ac03830e3ca7eecec3d6596c4771d34ae6799549 $
+ * $Id: 03005b43406169aa4c332530f338a24a65fd976b $
  *
  * Authors: Laurent Aimar <fenrir@videolan.org>
  *
@@ -78,14 +78,12 @@ static void Flush( decoder_t * );
 struct decoder_sys_t
 {
     mtime_t        i_max_stop;
-    int            i_scale;
 
     /* The following fields of decoder_sys_t are shared between decoder and spu units */
     vlc_mutex_t    lock;
     int            i_refcount;
 
     /* */
-    vlc_object_t *p_callbackobj;
     ASS_Library    *p_library;
     ASS_Renderer   *p_renderer;
     video_format_t fmt;
@@ -128,19 +126,6 @@ typedef struct
 static int BuildRegions( rectangle_t *p_region, int i_max_region, ASS_Image *p_img_list, int i_width, int i_height );
 static void RegionDraw( subpicture_region_t *p_region, ASS_Image *p_img );
 
-static int ScalingCallback (vlc_object_t *obj, const char *var,
-                            vlc_value_t old, vlc_value_t cur, void *data)
-{
-    decoder_sys_t *p_sys = data;
-    VLC_UNUSED(obj);
-    VLC_UNUSED(var);
-    VLC_UNUSED(old);
-    vlc_mutex_lock(&p_sys->lock);
-    p_sys->i_scale = cur.i_int;
-    vlc_mutex_unlock(&p_sys->lock);
-    return VLC_SUCCESS;
-}
-
 //#define DEBUG_REGION
 
 /*****************************************************************************
@@ -166,7 +151,6 @@ static int Create( vlc_object_t *p_this )
     p_sys->i_refcount = 1;
     memset( &p_sys->fmt, 0, sizeof(p_sys->fmt) );
     p_sys->i_max_stop = VLC_TS_INVALID;
-    p_sys->i_scale = var_InheritInteger( p_dec, "sub-text-scale" );
     p_sys->p_library  = NULL;
     p_sys->p_renderer = NULL;
     p_sys->p_track    = NULL;
@@ -298,18 +282,6 @@ static int Create( vlc_object_t *p_this )
     }
     ass_process_codec_private( p_track, p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra );
 
-    vlc_value_t val;
-    for ( vlc_object_t *obj = p_dec; obj; obj = obj->obj.parent )
-    {
-        if( var_GetChecked( obj,  "sub-text-scale" ,
-                            VLC_VAR_INTEGER, &val ) == VLC_SUCCESS )
-        {
-            var_AddCallback( obj, "sub-text-scale", ScalingCallback, p_sys );
-            p_sys->p_callbackobj = obj;
-            break;
-        }
-    }
-
     p_dec->fmt_out.i_codec = VLC_CODEC_RGBA;
 
     return VLC_SUCCESS;
@@ -342,11 +314,6 @@ static void DecSysRelease( decoder_sys_t *p_sys )
         return;
     }
     vlc_mutex_unlock( &p_sys->lock );
-
-    if( p_sys->p_callbackobj )
-        var_DelCallback( p_sys->p_callbackobj, "sub-text-scale",
-                         ScalingCallback, p_sys );
-
     vlc_mutex_destroy( &p_sys->lock );
 
     if( p_sys->p_track )
@@ -473,14 +440,15 @@ static int SubpictureValidate( subpicture_t *p_subpic,
     if( b_fmt_src || b_fmt_dst )
     {
         ass_set_frame_size( p_sys->p_renderer, fmt.i_visible_width, fmt.i_visible_height );
+#if LIBASS_VERSION > 0x01010000
+        ass_set_storage_size( p_sys->p_renderer, p_fmt_src->i_visible_width, p_fmt_src->i_visible_height );
+#endif
         const double src_ratio = (double)p_fmt_src->i_visible_width / p_fmt_src->i_visible_height;
         const double dst_ratio = (double)p_fmt_dst->i_visible_width / p_fmt_dst->i_visible_height;
         ass_set_aspect_ratio( p_sys->p_renderer, dst_ratio / src_ratio, 1 );
         p_sys->fmt = fmt;
     }
 
-    ass_set_font_scale( p_sys->p_renderer, p_sys->i_scale / 100.0 );
-
     /* */
     const mtime_t i_stream_date = p_subpic->updater.p_sys->i_pts + (i_ts - p_subpic->i_start);
     int i_changed;
