@@ -44,6 +44,7 @@
 
 #include <algorithm>
 #include <ctime>
+#include <cassert>
 
 using namespace adaptive::http;
 using namespace adaptive::logic;
@@ -69,8 +70,6 @@ PlaylistManager::PlaylistManager( demux_t *p_demux_,
     b_canceled = false;
     b_preparsing = false;
     nextPlaylistupdate = 0;
-    demux.i_nzpcr = VLC_TS_INVALID;
-    demux.i_firstpcr = VLC_TS_INVALID;
     demux.pcr_syncpoint = TimestampSynchronizationPoint::RandomAccess;
     vlc_mutex_init(&demux.lock);
     vlc_cond_init(&demux.cond);
@@ -126,12 +125,13 @@ bool PlaylistManager::setupPeriod()
         if(set && streamFactory)
         {
             SegmentTracker *tracker = new SegmentTracker(resources, logic,
-                                                         bufferingLogic, set);
+                                                         bufferingLogic, set,
+                                                         &synchronizationReferences);
             if(!tracker)
                 continue;
 
             AbstractStream *st = streamFactory->create(p_demux, set->getStreamFormat(),
-                                                       tracker, resources->getConnManager());
+                                                       tracker);
             if(!st)
             {
                 delete tracker;
@@ -219,7 +219,7 @@ static bool streamCompare(const PrioritizedAbstractStream &a,  const Prioritized
     return false;
 }
 
-AbstractStream::BufferingStatus PlaylistManager::bufferize(mtime_t i_nzdeadline,
+AbstractStream::BufferingStatus PlaylistManager::bufferize(Times deadline,
                                                            mtime_t i_min_buffering,
                                                            mtime_t i_max_buffering,
                                                            mtime_t i_target_buffering)
@@ -234,8 +234,7 @@ AbstractStream::BufferingStatus PlaylistManager::bufferize(mtime_t i_nzdeadline,
     {
         PrioritizedAbstractStream &p = *it;
         p.st = *sit;
-        p.status = p.st->getLastBufferStatus();
-        p.demuxed_amount = p.st->getDemuxedAmount(i_nzdeadline);
+        p.status = p.st->getBufferAndStatus(deadline, i_min_buffering, i_max_buffering, &p.demuxed_amount);
         ++it;
     }
     std::sort(prioritized_streams.begin(), prioritized_streams.end(), streamCompare);
@@ -258,7 +257,7 @@ AbstractStream::BufferingStatus PlaylistManager::bufferize(mtime_t i_nzdeadline,
             /* initial */
         }
 
-        AbstractStream::BufferingStatus i_ret = st->bufferize(i_nzdeadline,
+        AbstractStream::BufferingStatus i_ret = st->bufferize(deadline,
                                                                i_min_buffering,
                                                                i_max_buffering,
                                                                i_target_buffering,
@@ -275,60 +274,63 @@ AbstractStream::BufferingStatus PlaylistManager::bufferize(mtime_t i_nzdeadline,
     }
 
     vlc_mutex_lock(&demux.lock);
-    if(demux.i_nzpcr == VLC_TS_INVALID &&
+    if(demux.times.continuous == VLC_TS_INVALID &&
         /* don't wait minbuffer on simple discontinuity or restart */
        (demux.pcr_syncpoint == TimestampSynchronizationPoint::Discontinuity ||
         /* prevents starting before buffering is reached */
         i_return != AbstractStream::BufferingStatus::Lessthanmin ))
     {
-        demux.i_nzpcr = getFirstDTS();
+        demux.times = getFirstTimes();
     }
     vlc_mutex_unlock(&demux.lock);
 
     return i_return;
 }
 
-AbstractStream::Status PlaylistManager::dequeue(mtime_t i_floor, mtime_t *pi_nzbarrier)
+AbstractStream::Status PlaylistManager::dequeue(Times floor, Times *barrier)
 {
     AbstractStream::Status i_return = AbstractStream::Status::Eof;
 
-    const mtime_t i_nzdeadline = *pi_nzbarrier;
+    const Times deadline = *barrier;
 
     std::vector<AbstractStream *>::iterator it;
     for(it=streams.begin(); it!=streams.end(); ++it)
     {
         AbstractStream *st = *it;
-        mtime_t i_pcr;
-        AbstractStream::Status i_ret = st->dequeue(i_nzdeadline, &i_pcr);
+        Times pcr;
+        AbstractStream::Status i_ret = st->dequeue(deadline, &pcr);
         if( i_ret > i_return )
             i_return = i_ret;
 
-        if( i_pcr > i_floor )
-            *pi_nzbarrier = std::min( *pi_nzbarrier, i_pcr - VLC_TS_0 );
+        if( pcr.continuous > floor.continuous )
+        {
+            if( barrier->continuous > pcr.continuous )
+                *barrier = pcr;
+        }
     }
 
     return i_return;
 }
 
-mtime_t PlaylistManager::getResumeTime() const
+StreamPosition PlaylistManager::getResumePosition() const
 {
-    vlc_mutex_lock(const_cast<vlc_mutex_t *>(&demux.lock));
-    mtime_t pcr = demux.i_nzpcr;
-    vlc_mutex_unlock(const_cast<vlc_mutex_t *>(&demux.lock));
-    return pcr;
+    vlc_mutex_locker locker(&demux.lock);
+    StreamPosition pos;
+    pos.times = demux.times;
+    return pos;
 }
 
-mtime_t PlaylistManager::getFirstDTS() const
+Times PlaylistManager::getFirstTimes() const
 {
-    mtime_t mindts = VLC_TS_INVALID;
-    std::vector<AbstractStream *>::const_iterator it;
-    for(it=streams.begin(); it!=streams.end(); ++it)
+    Times mindts;
+    for(const AbstractStream *stream : streams)
     {
-        const mtime_t dts = (*it)->getFirstDTS();
-        if(mindts == VLC_TS_INVALID)
+        const Times dts = stream->getFirstTimes();
+        if(mindts.continuous == VLC_TS_INVALID)
+            mindts = dts;
+        else if(dts.continuous != VLC_TS_INVALID &&
+                dts.continuous < mindts.continuous)
             mindts = dts;
-        else if(dts > VLC_TS_INVALID)
-            mindts = std::min(mindts, dts);
     }
     return mindts;
 }
@@ -345,10 +347,17 @@ unsigned PlaylistManager::getActiveStreamsCount() const
     return count;
 }
 
-bool PlaylistManager::setPosition(mtime_t time)
+bool PlaylistManager::setPosition(mtime_t mediatime, double pos, bool accurate)
 {
     bool ret = true;
     bool hasValidStream = false;
+    StreamPosition streampos;
+    streampos.times = demux.firsttimes;
+    if(streampos.times.continuous != VLC_TS_INVALID)
+        streampos.times.offsetBy(mediatime - streampos.times.segment.media);
+    else
+        streampos.times.segment.media = mediatime;
+    streampos.pos = pos;
     for(int real = 0; real < 2; real++)
     {
         /* Always probe if we can seek first */
@@ -359,7 +368,7 @@ bool PlaylistManager::setPosition(mtime_t time)
             if(st->isValid() && !st->isDisabled())
             {
                 hasValidStream = true;
-                ret &= st->setPosition(time, !real);
+                ret &= st->setPosition(streampos, !real);
             }
         }
         if(!ret)
@@ -370,6 +379,15 @@ bool PlaylistManager::setPosition(mtime_t time)
         msg_Warn(p_demux, "there is no valid streams");
         ret = false;
     }
+
+    if(accurate && ret && streampos.times.continuous >= VLC_TS_0)
+    {
+        es_out_Control(p_demux->out, ES_OUT_SET_NEXT_DISPLAY_TIME,
+                       streampos.times.continuous);
+        SeekDebug(msg_Dbg(p_demux,"ES_OUT_SET_NEXT_DISPLAY_TIME to %" PRId64,
+                          streampos.times.continuous));
+    }
+
     return ret;
 }
 
@@ -404,10 +422,10 @@ bool PlaylistManager::updatePlaylist()
     return true;
 }
 
-mtime_t PlaylistManager::getCurrentDemuxTime() const
+Times PlaylistManager::getTimes(bool b_first) const
 {
-    vlc_mutex_locker locker(const_cast<vlc_mutex_t *>(&demux.lock));
-    return demux.i_nzpcr;
+    vlc_mutex_locker locker(&demux.lock);
+    return b_first ? demux.firsttimes : demux.times;
 }
 
 mtime_t PlaylistManager::getMinAheadTime() const
@@ -427,7 +445,7 @@ mtime_t PlaylistManager::getMinAheadTime() const
 
 bool PlaylistManager::reactivateStream(AbstractStream *stream)
 {
-    return stream->reactivate(getResumeTime());
+    return stream->reactivate(getResumePosition());
 }
 
 #define DEMUX_INCREMENT (CLOCK_FREQ / 20)
@@ -442,7 +460,7 @@ int PlaylistManager::demux_callback(demux_t *p_demux)
 int PlaylistManager::doDemux(int64_t increment)
 {
     vlc_mutex_lock(&demux.lock);
-    if(demux.i_nzpcr == VLC_TS_INVALID)
+    if(demux.times.continuous == VLC_TS_INVALID)
     {
         bool b_dead = true;
         bool b_all_disabled = true;
@@ -458,13 +476,20 @@ int PlaylistManager::doDemux(int64_t increment)
         return (b_dead || b_all_disabled) ? VLC_DEMUXER_EOF : VLC_DEMUXER_SUCCESS;
     }
 
-    if(demux.i_firstpcr == VLC_TS_INVALID)
-        demux.i_firstpcr = demux.i_nzpcr;
+    Times barrier = demux.times;
+    barrier.offsetBy(increment);
 
-    mtime_t i_nzbarrier = demux.i_nzpcr + increment;
     vlc_mutex_unlock(&demux.lock);
 
-    AbstractStream::Status status = dequeue(demux.i_nzpcr, &i_nzbarrier);
+    AbstractStream::Status status = dequeue(demux.times, &barrier);
+
+    vlc_mutex_lock(&demux.lock);
+    if(demux.firsttimes.continuous == VLC_TS_INVALID && barrier.continuous != VLC_TS_INVALID)
+    {
+        demux.firsttimes = barrier;
+        assert(barrier.segment.media != VLC_TS_INVALID);
+    }
+    vlc_mutex_unlock(&demux.lock);
 
     updateControlsPosition();
 
@@ -484,8 +509,8 @@ int PlaylistManager::doDemux(int64_t increment)
                 if (!setupPeriod())
                     return VLC_DEMUXER_EOF;
 
-                demux.i_nzpcr = VLC_TS_INVALID;
-                demux.i_firstpcr = VLC_TS_INVALID;
+                demux.times = Times();
+                demux.firsttimes = Times();
                 es_out_Control(p_demux->out, ES_OUT_RESET_PCR);
 
                 setBufferingRunState(true);
@@ -499,18 +524,18 @@ int PlaylistManager::doDemux(int64_t increment)
         break;
     case AbstractStream::Status::Discontinuity:
         vlc_mutex_lock(&demux.lock);
-        demux.i_nzpcr = VLC_TS_INVALID;
-        demux.i_firstpcr = VLC_TS_INVALID;
+        demux.times = Times();
+        demux.firsttimes = Times();
         demux.pcr_syncpoint = TimestampSynchronizationPoint::Discontinuity;
         es_out_Control(p_demux->out, ES_OUT_RESET_PCR);
         vlc_mutex_unlock(&demux.lock);
         break;
     case AbstractStream::Status::Demuxed:
         vlc_mutex_lock(&demux.lock);
-        if( demux.i_nzpcr != VLC_TS_INVALID && i_nzbarrier != demux.i_nzpcr )
+        if( demux.times.continuous != VLC_TS_INVALID && barrier.continuous != demux.times.continuous )
         {
-            demux.i_nzpcr = i_nzbarrier;
-            mtime_t pcr = VLC_TS_0 + std::max(INT64_C(0), demux.i_nzpcr - CLOCK_FREQ / 10);
+            demux.times = barrier;
+            mtime_t pcr = VLC_TS_0 + std::max(INT64_C(0), demux.times.continuous - CLOCK_FREQ/10);
             es_out_Control(p_demux->out, ES_OUT_SET_GROUP_PCR, 0, pcr);
         }
         vlc_mutex_unlock(&demux.lock);
@@ -551,7 +576,7 @@ int PlaylistManager::doControl(int i_query, va_list args)
             if(playlist->isLive())
             {
                 mtime_t now = mdate();
-                demux.i_nzpcr = VLC_TS_INVALID;
+                demux.times = Times();
                 cached.lastupdate = 0;
                 if(b_pause)
                 {
@@ -609,20 +634,23 @@ int PlaylistManager::doControl(int i_query, va_list args)
             }
 
             double pos = va_arg(args, double);
+            bool accurate = va_arg(args, int);
             mtime_t seekTime = cached.playlistStart + cached.playlistLength * pos;
 
             SeekDebug(msg_Dbg(p_demux, "Seek %f to %ld plstart %ld duration %ld",
                    pos, seekTime, cached.playlistEnd, cached.playlistLength));
 
-            if(!setPosition(seekTime))
+            if(!setPosition(seekTime, pos, accurate))
             {
                 setBufferingRunState(true);
                 return VLC_EGENERIC;
             }
 
             demux.pcr_syncpoint = TimestampSynchronizationPoint::RandomAccess;
-            demux.i_nzpcr = VLC_TS_INVALID;
+            demux.times = Times();
+            demux.firsttimes = Times();
             cached.lastupdate = 0;
+            cached.i_time = VLC_TS_INVALID;
             setBufferingRunState(true);
             break;
         }
@@ -631,8 +659,9 @@ int PlaylistManager::doControl(int i_query, va_list args)
         {
             setBufferingRunState(false); /* stop downloader first */
 
-            mtime_t time = va_arg(args, int64_t);
-            if(!setPosition(time))
+            mtime_t time = va_arg(args, mtime_t);
+            bool accurate = va_arg(args, int);
+            if(!setPosition(time, -1, accurate))
             {
                 setBufferingRunState(true);
                 return VLC_EGENERIC;
@@ -640,8 +669,10 @@ int PlaylistManager::doControl(int i_query, va_list args)
 
             vlc_mutex_locker locker(&cached.lock);
             demux.pcr_syncpoint = TimestampSynchronizationPoint::RandomAccess;
-            demux.i_nzpcr = VLC_TS_INVALID;
+            demux.times = Times();
+            demux.firsttimes = Times();
             cached.lastupdate = 0;
+            cached.i_time = VLC_TS_INVALID;
             setBufferingRunState(true);
             break;
         }
@@ -686,10 +717,10 @@ void PlaylistManager::Run()
         }
 
         vlc_mutex_lock(&demux.lock);
-        mtime_t i_nzpcr = demux.i_nzpcr;
+        Times pcr = demux.times;
         vlc_mutex_unlock(&demux.lock);
 
-        AbstractStream::BufferingStatus i_return = bufferize(i_nzpcr, i_min_buffering,
+        AbstractStream::BufferingStatus i_return = bufferize(pcr, i_min_buffering,
                                                              i_max_buffering, i_target_buffering);
 
         if(i_return != AbstractStream::BufferingStatus::Lessthanmin)
@@ -734,17 +765,12 @@ void PlaylistManager::updateControlsPosition()
         return;
     cached.lastupdate = now;
 
-    mtime_t rapPlaylistStart = 0;
-    mtime_t rapDemuxStart = 0;
-    std::vector<AbstractStream *>::iterator it;
-    for(it=streams.begin(); it!=streams.end(); ++it)
+    for(AbstractStream* st : streams)
     {
-        AbstractStream *st = *it;
         if(st->isValid() && !st->isDisabled() && st->isSelected())
         {
             if(st->getMediaPlaybackTimes(&cached.playlistStart, &cached.playlistEnd,
-                                         &cached.playlistLength,
-                                         &rapPlaylistStart, &rapDemuxStart))
+                                         &cached.playlistLength))
                 break;
         }
     }
@@ -763,19 +789,20 @@ void PlaylistManager::updateControlsPosition()
      * All seeks need to be done in playlist time !
      */
 
-    mtime_t currentDemuxTime = getCurrentDemuxTime();
+    SeekDebug(Times startTimes = getTimes(true));
+    Times currentTimes = getTimes();
     cached.b_live = playlist->isLive();
 
     SeekDebug(msg_Dbg(p_demux, "playlist Start/End %ld/%ld len %ld"
                                "rap pl/demux (%ld/%ld)",
                       cached.playlistStart, cached.playlistEnd, cached.playlistEnd,
-                      rapPlaylistStart, rapDemuxStart));
+                      startTimes.segment.media, startTimes.segment.demux));
 
     if(cached.b_live)
     {
         /* Special case for live until we can provide relative start to fully match
            the above description */
-        cached.i_time = currentDemuxTime;
+        cached.i_time = currentTimes.segment.media;
 
         if(cached.playlistStart != cached.playlistEnd)
         {
@@ -785,11 +812,11 @@ void PlaylistManager::updateControlsPosition()
                 cached.playlistStart = cached.playlistEnd - cached.playlistLength;
             }
         }
-        const mtime_t currentTime = getCurrentDemuxTime();
-        if(currentTime > cached.playlistStart &&
-           currentTime <= cached.playlistEnd && cached.playlistLength)
+
+        if(cached.i_time > VLC_TS_0 + cached.playlistStart &&
+           cached.i_time <= VLC_TS_0 + cached.playlistEnd && cached.playlistLength)
         {
-            cached.f_position = ((double)(currentTime - cached.playlistStart)) / cached.playlistLength;
+            cached.f_position = ((double)(cached.i_time - VLC_TS_0 - cached.playlistStart)) / cached.playlistLength;
         }
         else
         {
@@ -801,14 +828,10 @@ void PlaylistManager::updateControlsPosition()
         if(playlist->duration.Get() > cached.playlistLength)
             cached.playlistLength = playlist->duration.Get();
 
-        if(cached.playlistLength && currentDemuxTime)
+        if(cached.playlistLength && currentTimes.segment.media != VLC_TS_INVALID)
         {
-            /* convert to playlist time */
-            mtime_t rapRelOffset = currentDemuxTime - rapDemuxStart; /* offset from start/seek */
-            mtime_t absPlaylistTime = rapPlaylistStart + rapRelOffset; /* converted as abs playlist time */
-            mtime_t relMediaTime = absPlaylistTime - cached.playlistStart; /* elapsed, in playlist time */
-            cached.i_time = absPlaylistTime;
-            cached.f_position = (double) relMediaTime / cached.playlistLength;
+            cached.i_time = currentTimes.segment.media;
+            cached.f_position = (double) (cached.i_time - VLC_TS_0 - cached.playlistStart) / cached.playlistLength;
         }
         else
         {
@@ -816,8 +839,9 @@ void PlaylistManager::updateControlsPosition()
         }
     }
 
-    SeekDebug(msg_Dbg(p_demux, "cached.i_time (%ld) cur %ld rap start (pl %ld/dmx %ld)",
-               cached.i_time, currentDemuxTime, rapPlaylistStart, rapDemuxStart));
+    SeekDebug(msg_Dbg(p_demux, "cached.i_time (%ld) cur %ld rap start (pl %ld/dmx %ld) pos %f",
+                      cached.i_time, currentTimes.continuous, startTimes.segment.media,
+                            startTimes.segment.demux, cached.f_position));
 }
 
 AbstractAdaptationLogic *PlaylistManager::createLogic(AbstractAdaptationLogic::LogicType type, AbstractConnectionManager *conn)
