@@ -42,6 +42,8 @@ namespace adaptive
     using namespace playlist;
     using namespace logic;
 
+    using StreamPosition = AbstractStream::StreamPosition;
+
     class PlaylistManager
     {
         public:
@@ -57,9 +59,9 @@ namespace adaptive
             bool    started() const;
             void    stop();
 
-            AbstractStream::BufferingStatus bufferize(mtime_t, mtime_t,
+            AbstractStream::BufferingStatus bufferize(Times, mtime_t,
                                                       mtime_t, mtime_t);
-            AbstractStream::Status dequeue(mtime_t, mtime_t *);
+            AbstractStream::Status dequeue(Times, Times *);
 
             virtual bool needsUpdate() const;
             virtual bool updatePlaylist();
@@ -75,12 +77,12 @@ namespace adaptive
             virtual int doDemux(int64_t);
 
             void    setLivePause(bool);
-            virtual bool    setPosition(mtime_t);
-            mtime_t getResumeTime() const;
-            mtime_t getFirstDTS() const;
+            virtual bool setPosition(mtime_t, double pos = -1, bool accurate = false);
+            StreamPosition getResumePosition() const;
+            Times getFirstTimes() const;
             unsigned getActiveStreamsCount() const;
 
-            mtime_t getCurrentDemuxTime() const;
+            Times getTimes(bool = false) const;
             mtime_t getMinAheadTime() const;
 
             virtual bool reactivateStream(AbstractStream *);
@@ -114,9 +116,8 @@ namespace adaptive
             struct
             {
                 TimestampSynchronizationPoint pcr_syncpoint;
-                mtime_t     i_nzpcr;
-                mtime_t     i_firstpcr;
-                vlc_mutex_t lock;
+                Times times, firsttimes;
+                mutable vlc_mutex_t lock;
                 vlc_cond_t  cond;
             } demux;
 
@@ -137,6 +138,8 @@ namespace adaptive
                 time_t      lastupdate;
             } cached;
 
+            SynchronizationReferences synchronizationReferences;
+
         private:
             void setBufferingRunState(bool);
             void Run();
