@@ -35,38 +35,44 @@ extern "C"
 
 using namespace adaptive;
 
-StreamFormat::operator unsigned() const
+StreamFormat::operator StreamFormat::Type() const
 {
-    return formatid;
+    return type;
 }
 
 std::string StreamFormat::str() const
 {
-    switch(formatid)
+    switch(type)
     {
-        case MPEG2TS:
+        case Type::MPEG2TS:
             return "TS";
-        case MP4:
+        case Type::MP4:
             return "MP4";
-        case WEBVTT:
+        case Type::WebVTT:
             return "WebVTT";
-        case TTML:
+        case Type::TTML:
             return "Timed Text";
-        case PACKEDAAC:
+        case Type::PackedAAC:
             return "Packed AAC";
-        case WEBM:
+        case Type::PackedMP3:
+            return "Packed MP3";
+        case Type::PackedAC3:
+            return "Packed AC-3";
+        case Type::WebM:
             return "WebM";
-        case UNSUPPORTED:
+        case Type::Ogg:
+            return "Ogg";
+        case Type::Unsupported:
             return "Unsupported";
         default:
-        case UNKNOWN:
+        case Type::Unknown:
             return "Unknown";
     }
 }
 
-StreamFormat::StreamFormat( unsigned formatid_ )
+StreamFormat::StreamFormat( Type type_ )
 {
-    formatid = formatid_;
+    type = type_;
 }
 
 StreamFormat::StreamFormat( const std::string &mimetype )
@@ -74,22 +80,26 @@ StreamFormat::StreamFormat( const std::string &mimetype )
     std::string mime = mimetype;
     std::transform(mime.begin(), mime.end(), mime.begin(), ::tolower);
     std::string::size_type pos = mime.find("/");
-    formatid = UNKNOWN;
+    type = Type::Unknown;
     if(pos != std::string::npos)
     {
         std::string tail = mime.substr(pos + 1);
         if(tail == "mp4")
-            formatid = StreamFormat::MP4;
+            type = StreamFormat::Type::MP4;
         else if(tail == "aac")
-            formatid = StreamFormat::PACKEDAAC;
+            type = StreamFormat::Type::PackedAAC;
+        else if(tail == "mpeg" || tail == "mp3")
+            type = StreamFormat::Type::PackedMP3;
+        else if(tail == "ac3")
+            type = StreamFormat::Type::PackedAC3;
         else if (tail == "mp2t")
-            formatid = StreamFormat::MPEG2TS;
+            type = StreamFormat::Type::MPEG2TS;
         else if (tail == "vtt")
-            formatid = StreamFormat::WEBVTT;
+            type = StreamFormat::Type::WebVTT;
         else if (tail == "ttml+xml")
-            formatid = StreamFormat::TTML;
+            type = StreamFormat::Type::TTML;
         else if (tail == "webm")
-            formatid = StreamFormat::WEBM;
+            type = StreamFormat::Type::WebM;
     }
 }
 
@@ -98,30 +108,46 @@ static int ID3Callback(uint32_t, const uint8_t *, size_t, void *)
     return VLC_EGENERIC;
 }
 
+static bool IsWebVTT(const char *p, size_t sz)
+{
+    /* match optional U+FEFF BOM */
+    const uint8_t webvtt[] = { 0xEF, 0xBB, 0xBF, 0x57, 0x45, 0x42, 0x56, 0x54, 0x54 };
+    for(int i=3; i>=0; i-=3)
+    {
+        if(sz > (size_t)(10 - i) &&
+           !memcmp(webvtt + i, p, 9 - i) &&
+           (p[9 - i] == '\n' || p[9 - i] == '\r' || p[9 - i] == ' ' || p[9 - i] == '\t'))
+            return true;
+    }
+    return false;
+}
+
 StreamFormat::StreamFormat(const void *data_, size_t sz)
 {
     const uint8_t *data = reinterpret_cast<const uint8_t *>(data_);
-    formatid = UNKNOWN;
-    const char moov[] = "ftypmoovmoof";
+    type = Type::Unknown;
+    const char moov[] = "ftypmoovmoofemsg";
 
     if(sz > 188 && data[0] == 0x47 && data[188] == 0x47)
-        formatid = StreamFormat::MPEG2TS;
+        type = StreamFormat::Type::MPEG2TS;
     else if(sz > 8 && (!memcmp(&moov,    &data[4], 4) ||
                        !memcmp(&moov[4], &data[4], 4) ||
-                       !memcmp(&moov[8], &data[4], 4)))
-        formatid = StreamFormat::MP4;
-    else if(sz > 7 && !memcmp("WEBVTT", data, 6) &&
-            std::isspace(static_cast<unsigned char>(data[7])))
-        formatid = StreamFormat::WEBVTT;
+                       !memcmp(&moov[8], &data[4], 4) ||
+                       !memcmp(&moov[12], &data[4], 4)))
+        type = StreamFormat::Type::MP4;
+    else if(IsWebVTT((const char *)data, sz))
+        type = StreamFormat::Type::WebVTT;
     else if(sz > 4 && !memcmp("\x1A\x45\xDF\xA3", data, 4))
-        formatid = StreamFormat::WEBM;
+        type = StreamFormat::Type::WebM;
+    else if(sz > 4 && !memcmp("OggS", data, 4))
+        type = StreamFormat::Type::Ogg;
     else /* Check Packet Audio formats */
     {
         /* It MUST have ID3 header, but HLS spec is an oxymoron */
-        if(sz > 10 && ID3TAG_IsTag(data, false))
+        while(sz > 10 && ID3TAG_IsTag(data, false))
         {
             size_t tagsize = ID3TAG_Parse(data, sz, ID3Callback, this);
-            if(tagsize >= sz)
+            if(tagsize >= sz || tagsize == 0)
                 return; /* not enough peek */
             data += tagsize;
             sz -= tagsize;
@@ -130,7 +156,15 @@ StreamFormat::StreamFormat(const void *data_, size_t sz)
         if(sz > 3 && (!memcmp("\xFF\xF1", data, 2) ||
                       !memcmp("\xFF\xF9", data, 2)))
         {
-            formatid = StreamFormat::PACKEDAAC;
+            type = StreamFormat::Type::PackedAAC;
+        }
+        else if(sz > 4 && data[0] == 0xFF && (data[1] & 0xE6) > 0xE0)
+        {
+            type = StreamFormat::Type::PackedMP3;
+        }
+        else if(sz > 4 && data[0] == 0x0b && data[1] == 0x77)
+        {
+            type = StreamFormat::Type::PackedAC3;
         }
     }
 }
@@ -140,12 +174,22 @@ StreamFormat::~StreamFormat()
 
 }
 
+bool StreamFormat::operator ==(Type t) const
+{
+    return type == t;
+}
+
+bool StreamFormat::operator !=(Type t) const
+{
+    return type != t;
+}
+
 bool StreamFormat::operator ==(const StreamFormat &other) const
 {
-    return formatid == other.formatid;
+    return type == other.type;
 }
 
 bool StreamFormat::operator !=(const StreamFormat &other) const
 {
-    return formatid != other.formatid;
+    return type != other.type;
 }
