@@ -37,24 +37,24 @@ namespace adaptive
         class BasePeriod : public SegmentInformation
         {
             public:
-                BasePeriod(AbstractPlaylist *);
+                BasePeriod(BasePlaylist *);
                 virtual ~BasePeriod ();
 
                 const std::vector<BaseAdaptationSet *>& getAdaptationSets   () const;
-                BaseAdaptationSet *                 getAdaptationSetByID(const ID &);
+                BaseAdaptationSet *                 getAdaptationSetByID(const ID &) const;
                 void                                addAdaptationSet    (BaseAdaptationSet *AdaptationSet);
                 void                                debug               (vlc_object_t *,int = 0) const;
 
-                virtual mtime_t getPeriodStart() const; /* reimpl */
-                virtual mtime_t getPeriodDuration() const;
-                virtual AbstractPlaylist *getPlaylist() const; /* reimpl */
+                virtual mtime_t getPeriodStart() const override;
+                virtual mtime_t getPeriodDuration() const override;
+                virtual BasePlaylist *getPlaylist() const override;
 
                 Property<mtime_t> duration;
                 Property<mtime_t> startTime;
 
             private:
                 std::vector<BaseAdaptationSet *>    adaptationSets;
-                AbstractPlaylist *playlist;
+                BasePlaylist *playlist;
         };
     }
 }
