@@ -21,6 +21,7 @@
 #define COMMANDSQUEUE_HPP_
 
 #include "FakeESOutID.hpp"
+#include "../Time.hpp"
 
 #include <vlc_common.h>
 #include <vlc_es.h>
@@ -38,11 +39,12 @@ namespace adaptive
         public:
             virtual ~AbstractCommand();
             virtual void Execute( ) = 0;
-            virtual mtime_t getTime() const;
+            virtual const Times & getTimes() const;
             int getType() const;
 
         protected:
             AbstractCommand( int );
+            Times times;
             int type;
     };
 
@@ -62,10 +64,9 @@ namespace adaptive
         public:
             virtual ~EsOutSendCommand();
             virtual void Execute( ) override;
-            virtual mtime_t getTime() const override;
 
         protected:
-            EsOutSendCommand( AbstractFakeESOutID *, block_t * );
+            EsOutSendCommand( AbstractFakeESOutID *, const SegmentTimes &, block_t * );
             block_t *p_block;
     };
 
@@ -95,12 +96,10 @@ namespace adaptive
         friend class CommandsFactory;
         public:
             virtual void Execute() override;
-            virtual mtime_t getTime() const override;
 
         protected:
-            EsOutControlPCRCommand( int, mtime_t );
+            EsOutControlPCRCommand( int, const SegmentTimes &, mtime_t );
             int group;
-            mtime_t pcr;
     };
 
     class EsOutDestroyCommand : public AbstractCommand
@@ -148,19 +147,34 @@ namespace adaptive
             AbstractFakeEsOut *out;
     };
 
+    class EsOutMediaProgressCommand : public AbstractCommand
+    {
+         friend class CommandsFactory;
+        public:
+            virtual void Execute() override;
+
+        protected:
+            EsOutMediaProgressCommand( const SegmentTimes & );
+    };
+
     /* Factory so we can alter behaviour and filter on execution */
     class CommandsFactory
     {
         public:
             virtual ~CommandsFactory() {}
-            virtual EsOutSendCommand * createEsOutSendCommand( AbstractFakeESOutID *, block_t * ) const;
+            virtual EsOutSendCommand * createEsOutSendCommand( AbstractFakeESOutID *,
+                                                               const SegmentTimes &,
+                                                               block_t * ) const;
             virtual EsOutDelCommand * createEsOutDelCommand( AbstractFakeESOutID * ) const;
             virtual EsOutAddCommand * createEsOutAddCommand( AbstractFakeESOutID * ) const;
-            virtual EsOutControlPCRCommand * createEsOutControlPCRCommand( int, mtime_t ) const;
+            virtual EsOutControlPCRCommand * createEsOutControlPCRCommand( int,
+                                                                           const SegmentTimes &,
+                                                                           mtime_t ) const;
             virtual EsOutControlResetPCRCommand * creatEsOutControlResetPCRCommand() const;
             virtual EsOutDestroyCommand * createEsOutDestroyCommand() const;
             virtual EsOutMetaCommand * createEsOutMetaCommand( AbstractFakeEsOut *, int, const vlc_meta_t * ) const;
             virtual EsOutMilestoneCommand * createEsOutMilestoneCommand( AbstractFakeEsOut * ) const;
+            virtual EsOutMediaProgressCommand * createEsOutMediaProgressCommand( const SegmentTimes & ) const;
     };
 
     using Queueentry = std::pair<uint64_t, AbstractCommand *>;
@@ -171,7 +185,7 @@ namespace adaptive
             AbstractCommandsQueue();
             virtual ~AbstractCommandsQueue() = default;
             virtual void Schedule( AbstractCommand *, EsType = EsType::Other ) = 0;
-            virtual mtime_t Process( mtime_t )  = 0;
+            virtual Times Process( Times )  = 0;
             virtual void Abort( bool b_reset )  = 0;
             virtual void Commit()  = 0;
             virtual bool isEmpty() const = 0;
@@ -180,10 +194,11 @@ namespace adaptive
             void setEOF( bool );
             bool isDraining() const;
             bool isEOF() const;
-            virtual mtime_t getDemuxedAmount(mtime_t) const  = 0;
-            virtual mtime_t getBufferingLevel() const  = 0;
-            virtual mtime_t getFirstDTS() const  = 0;
-            virtual mtime_t getPCR() const = 0;
+            virtual Times getDemuxedAmount(Times) const  = 0;
+            virtual Times getDemuxedMediaAmount(const Times &) const = 0;
+            virtual Times getBufferingLevel() const  = 0;
+            virtual Times getFirstTimes() const  = 0;
+            virtual Times getPCR() const = 0;
 
         protected:
             bool b_draining;
@@ -198,23 +213,25 @@ namespace adaptive
             CommandsQueue();
             virtual ~CommandsQueue();
             virtual void Schedule( AbstractCommand *, EsType = EsType::Other ) override;
-            virtual mtime_t Process( mtime_t ) override;
+            virtual Times Process( Times ) override;
             virtual void Abort( bool b_reset ) override;
             virtual void Commit() override;
             virtual bool isEmpty() const override;
             virtual void setDraining() override;
-            virtual mtime_t getDemuxedAmount(mtime_t) const override;
-            virtual mtime_t getBufferingLevel() const override;
-            virtual mtime_t getFirstDTS() const override;
-            virtual mtime_t getPCR() const override;
+            virtual Times getDemuxedAmount(Times) const override;
+            virtual Times getDemuxedMediaAmount(const Times &) const override;
+            virtual Times getBufferingLevel() const override;
+            virtual Times getFirstTimes() const override;
+            virtual Times getPCR() const override;
 
         private:
             void LockedCommit();
             void LockedSetDraining();
             std::list<Queueentry> incoming;
             std::list<Queueentry> commands;
-            mtime_t bufferinglevel;
-            mtime_t pcr;
+            SegmentTimes bufferinglevel_media;
+            Times bufferinglevel;
+            Times pcr;
             uint64_t nextsequence;
     };
 }
