@@ -129,18 +129,18 @@ FakeESOut * FakeESOut::LockedFakeEsOut::operator ->()
     return p;
 }
 
-FakeESOut::FakeESOut( es_out_t *es, CommandsQueue *queue )
+FakeESOut::FakeESOut( es_out_t *es, AbstractCommandsQueue *queue,
+                      CommandsFactory *cf )
     : AbstractFakeEsOut()
     , real_es_out( es )
-    , extrainfo( NULL )
+    , extrainfo( nullptr )
     , commandsqueue( queue )
+    , commandsfactory( cf )
     , timestamps_offset( 0 )
 {
     associated.b_timestamp_set = false;
     expected.b_timestamp_set = false;
-    timestamp_first = 0;
     priority = ES_PRIORITY_SELECTABLE_MIN;
-
     vlc_mutex_init(&lock);
 }
 
@@ -149,38 +149,36 @@ FakeESOut::LockedFakeEsOut FakeESOut::WithLock()
     return LockedFakeEsOut(*this);
 }
 
-CommandsQueue * FakeESOut::commandsQueue()
+AbstractCommandsQueue * FakeESOut::commandsQueue()
 {
     return commandsqueue;
 }
 
+CommandsFactory * FakeESOut::commandsFactory() const
+{
+    return commandsfactory;
+}
+
 FakeESOut::~FakeESOut()
 {
     recycleAll();
     gc();
 
     delete commandsqueue;
+    delete commandsfactory;
     vlc_mutex_destroy(&lock);
 }
 
 void FakeESOut::resetTimestamps()
 {
-    setExpectedTimestamp(-1);
-    setAssociatedTimestamp(-1);
-}
-
-bool FakeESOut::getStartTimestamps( mtime_t *pi_mediats, mtime_t *pi_demuxts )
-{
-    if(!expected.b_timestamp_set)
-        return false;
-    *pi_demuxts = timestamp_first;
-    *pi_mediats = expected.timestamp;
-    return true;
+    setExpectedTimestamp(VLC_TS_INVALID);
+    setAssociatedTimestamp(VLC_TS_INVALID);
+    startTimes = SegmentTimes();
 }
 
 void FakeESOut::setExpectedTimestamp(mtime_t ts)
 {
-    if(ts < 0)
+    if(ts == VLC_TS_INVALID)
     {
         expected.b_timestamp_set = false;
         timestamps_offset = 0;
@@ -195,7 +193,7 @@ void FakeESOut::setExpectedTimestamp(mtime_t ts)
 
 void FakeESOut::setAssociatedTimestamp(mtime_t ts)
 {
-    if(ts < 0)
+    if(ts == VLC_TS_INVALID)
     {
         associated.b_timestamp_set = false;
         timestamps_offset = 0;
@@ -208,6 +206,20 @@ void FakeESOut::setAssociatedTimestamp(mtime_t ts)
     }
 }
 
+void FakeESOut::setAssociatedTimestamp(mtime_t mpegts, mtime_t muxed)
+{
+    if(mpegts == VLC_TS_INVALID)
+    {
+        setAssociatedTimestamp(mpegts);
+    }
+    else
+    {
+        associated.b_timestamp_set = true;
+        associated.b_offset_calculated = true;
+        timestamps_offset = mpegts - muxed;
+    }
+}
+
 void FakeESOut::setExtraInfoProvider( ExtraFMTInfoInterface *extra )
 {
     extrainfo = extra;
@@ -233,10 +245,11 @@ FakeESOutID * FakeESOut::createNewID( const es_format_t *p_fmt )
     return es_id;
 }
 
-void FakeESOut::createOrRecycleRealEsID( FakeESOutID *es_id )
+void FakeESOut::createOrRecycleRealEsID( AbstractFakeESOutID *es_id_ )
 {
+    FakeESOutID *es_id = static_cast<FakeESOutID *>(es_id_);
     std::list<FakeESOutID *>::iterator it;
-    es_out_id_t *realid = NULL;
+    es_out_id_t *realid = nullptr;
 
     /* declared ES must are temporary until real ES decl */
     recycle_candidates.insert(recycle_candidates.begin(), declared.begin(), declared.end());
@@ -250,7 +263,7 @@ void FakeESOut::createOrRecycleRealEsID( FakeESOutID *es_id )
         if ( cand->isCompatible( es_id ) )
         {
             realid = cand->realESID();
-            cand->setRealESID( NULL );
+            cand->setRealESID( nullptr );
             delete *it;
             recycle_candidates.erase( it );
             break;
@@ -276,16 +289,35 @@ void FakeESOut::createOrRecycleRealEsID( FakeESOutID *es_id )
         realid = es_out_Add( real_es_out, &fmt );
         if( b_preexisting && b_select ) /* was previously selected on other format */
             es_out_Control( real_es_out, ES_OUT_SET_ES, realid );
+        es_format_Clean( &fmt );
     }
 
     es_id->setRealESID( realid );
 }
 
-void FakeESOut::setPriority(int p)
+void FakeESOut::setPriority( int p )
 {
     priority = p;
 }
 
+void FakeESOut::sendData( AbstractFakeESOutID *id_, block_t *p_block )
+{
+    FakeESOutID *id = static_cast<FakeESOutID *>(id_);
+    /* Be sure to notify Data before Sending, because UI would still not pick new ES */
+    gc();
+    if( !id->realESID() ||
+        es_out_Send( real_es_out, id->realESID(), p_block ) != VLC_SUCCESS )
+    {
+        block_Release( p_block );
+    }
+    gc();
+}
+
+void FakeESOut::sendMeta( int group, const vlc_meta_t *p_meta )
+{
+    es_out_Control( real_es_out, ES_OUT_SET_GROUP_META, group, p_meta );
+}
+
 size_t FakeESOut::esCount() const
 {
     if(!declared.empty())
@@ -299,9 +331,36 @@ size_t FakeESOut::esCount() const
     return i_count;
 }
 
+bool FakeESOut::hasSegmentStartTimes() const
+{
+    return startTimes.media != VLC_TS_INVALID;
+}
+
+void FakeESOut::setSegmentStartTimes(const SegmentTimes &t)
+{
+    startTimes = t;
+}
+
+void FakeESOut::setSegmentProgressTimes(const SegmentTimes &t)
+{
+    AbstractCommand *c = commandsFactory()->createEsOutMediaProgressCommand(t);
+    if(c)
+        commandsQueue()->Schedule(c);
+}
+
+bool FakeESOut::hasSynchronizationReference() const
+{
+    return synchronizationReference.second.continuous != VLC_TS_INVALID;
+}
+
+void FakeESOut::setSynchronizationReference(const SynchronizationReference &r)
+{
+    synchronizationReference = r;
+}
+
 void FakeESOut::schedulePCRReset()
 {
-    AbstractCommand *command = commandsqueue->factory()->creatEsOutControlResetPCRCommand();
+    AbstractCommand *command = commandsfactory->creatEsOutControlResetPCRCommand();
     if( likely(command) )
         commandsqueue->Schedule( command );
 }
@@ -314,7 +373,7 @@ void FakeESOut::scheduleAllForDeletion()
         FakeESOutID *es_id = *it;
         if(!es_id->scheduledForDeletion())
         {
-            AbstractCommand *command = commandsqueue->factory()->createEsOutDelCommand( es_id );
+            AbstractCommand *command = commandsfactory->createEsOutDelCommand( es_id );
             if( likely(command) )
             {
                 commandsqueue->Schedule( command );
@@ -385,12 +444,29 @@ bool FakeESOut::restarting() const
     return b;
 }
 
-void FakeESOut::recycle( FakeESOutID *id )
+void FakeESOut::recycle( AbstractFakeESOutID *id_ )
 {
+    FakeESOutID *id = static_cast<FakeESOutID *>(id_);
     fakeesidlist.remove( id );
     recycle_candidates.push_back( id );
 }
 
+void FakeESOut::milestoneReached()
+{
+    gc();
+}
+
+void FakeESOut::scheduleNecessaryMilestone()
+{
+    if( b_in_commands_group )
+    {
+        AbstractCommand *command = commandsfactory->createEsOutMilestoneCommand( this );
+         if( likely(command) )
+             commandsqueue->Schedule( command );
+         b_in_commands_group = false;
+    }
+}
+
 mtime_t FakeESOut::fixTimestamp(mtime_t ts)
 {
     if(ts != VLC_TS_INVALID)
@@ -404,7 +480,6 @@ mtime_t FakeESOut::fixTimestamp(mtime_t ts)
             {
                 timestamps_offset = associated.timestamp - ts;
                 associated.b_offset_calculated = true;
-                timestamp_first = ts + timestamps_offset;
             }
         }
         else if(expected.b_timestamp_set)
@@ -419,7 +494,6 @@ mtime_t FakeESOut::fixTimestamp(mtime_t ts)
                 else
                     timestamps_offset = 0;
                 expected.b_offset_calculated = true;
-                timestamp_first = ts + timestamps_offset;
             }
         }
         ts += timestamps_offset;
@@ -427,6 +501,48 @@ mtime_t FakeESOut::fixTimestamp(mtime_t ts)
     return ts;
 }
 
+mtime_t FakeESOut::applyTimestampContinuity(mtime_t ts)
+{
+    if(ts == VLC_TS_INVALID)
+        return ts;
+
+    constexpr mtime_t rollover = INT64_C(0x1FFFFFFFF) * 100 / 9;
+    constexpr mtime_t halfroll = INT64_C(0x0FFFFFFFF) * 100 / 9;
+    if(synchronizationReference.second.segment.demux != VLC_TS_INVALID)
+    {
+        while(ts - synchronizationReference.second.segment.demux > halfroll)
+        {
+            ts -= rollover;
+        }
+        while(synchronizationReference.second.segment.demux - ts > halfroll)
+        {
+            ts += rollover;
+        }
+    }
+
+    if(synchronizationReference.second.segment.demux != VLC_TS_INVALID &&
+       synchronizationReference.second.continuous != VLC_TS_INVALID)
+    {
+        mtime_t continuityoffset = synchronizationReference.second.continuous -
+                                   synchronizationReference.second.segment.demux;
+
+        /* avoid triggering false roll when reference is 13 hours away */
+        if(ts - synchronizationReference.second.segment.demux > halfroll / 2)
+            synchronizationReference.second.offsetBy(halfroll / 2);
+
+        ts += continuityoffset;
+        assert(ts >= VLC_TS_INVALID);
+    }
+    else /* First synchronization point */
+    {
+        synchronizationReference.second.segment = startTimes;
+        synchronizationReference.second.segment.demux = ts;
+        synchronizationReference.second.continuous = ts;
+    }
+
+    return ts;
+}
+
 void FakeESOut::declareEs(const es_format_t *fmt)
 {
     /* Declared ES are only visible until stream data flows.
@@ -458,7 +574,7 @@ es_out_id_t * FakeESOut::esOutAdd(const es_format_t *p_fmt)
     vlc_mutex_locker locker(&lock);
 
     if( p_fmt->i_cat != VIDEO_ES && p_fmt->i_cat != AUDIO_ES && p_fmt->i_cat != SPU_ES )
-        return NULL;
+        return nullptr;
 
     /* Feed the slave demux/stream_Demux with FakeESOutID struct,
      * we'll create real ES later on main demux on execution */
@@ -466,11 +582,12 @@ es_out_id_t * FakeESOut::esOutAdd(const es_format_t *p_fmt)
     if( likely(es_id) )
     {
         assert(!es_id->scheduledForDeletion());
-        AbstractCommand *command = commandsqueue->factory()->createEsOutAddCommand( es_id );
+        AbstractCommand *command = commandsfactory->createEsOutAddCommand( es_id );
         if( likely(command) )
         {
             fakeesidlist.push_back(es_id);
             commandsqueue->Schedule( command );
+            b_in_commands_group = true;
             return reinterpret_cast<es_out_id_t *>(es_id);
         }
         else
@@ -478,20 +595,41 @@ es_out_id_t * FakeESOut::esOutAdd(const es_format_t *p_fmt)
             delete es_id;
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 int FakeESOut::esOutSend(es_out_id_t *p_es, block_t *p_block)
 {
     vlc_mutex_locker locker(&lock);
 
+    scheduleNecessaryMilestone();
+
     FakeESOutID *es_id = reinterpret_cast<FakeESOutID *>( p_es );
     assert(!es_id->scheduledForDeletion());
 
     p_block->i_dts = fixTimestamp( p_block->i_dts );
     p_block->i_pts = fixTimestamp( p_block->i_pts );
 
-    AbstractCommand *command = commandsqueue->factory()->createEsOutSendCommand( es_id, p_block );
+    if(!hasSynchronizationReference() && p_block->i_dts != VLC_TS_INVALID)
+    {
+        synchronizationReference.second.segment = startTimes;
+        synchronizationReference.second.continuous = p_block->i_dts;
+        synchronizationReference.second.segment.demux = p_block->i_dts;
+        assert(hasSynchronizationReference());
+    }
+
+    p_block->i_dts = applyTimestampContinuity( p_block->i_dts );
+    p_block->i_pts = applyTimestampContinuity( p_block->i_pts );
+
+    SegmentTimes times;
+    if(p_block->i_dts != VLC_TS_INVALID)
+    {
+        times = synchronizationReference.second.segment;
+        times.offsetBy(p_block->i_dts - times.demux);
+        assert(times.media != VLC_TS_INVALID);
+    }
+
+   AbstractCommand *command = commandsfactory->createEsOutSendCommand( es_id, times, p_block );
     if( likely(command) )
     {
         commandsqueue->Schedule( command );
@@ -505,18 +643,21 @@ void FakeESOut::esOutDel(es_out_id_t *p_es)
     vlc_mutex_locker locker(&lock);
 
     FakeESOutID *es_id = reinterpret_cast<FakeESOutID *>( p_es );
-    AbstractCommand *command = commandsqueue->factory()->createEsOutDelCommand( es_id );
+    AbstractCommand *command = commandsfactory->createEsOutDelCommand( es_id );
     if( likely(command) )
     {
         es_id->setScheduledForDeletion();
         commandsqueue->Schedule( command );
     }
+    b_in_commands_group = true;
 }
 
 int FakeESOut::esOutControl(int i_query, va_list args)
 {
     vlc_mutex_locker locker(&lock);
 
+    scheduleNecessaryMilestone();
+
     switch( i_query )
     {
         case ES_OUT_SET_PCR:
@@ -528,8 +669,21 @@ int FakeESOut::esOutControl(int i_query, va_list args)
             else
                 i_group = 0;
             mtime_t  pcr = va_arg( args, mtime_t );
-            pcr = fixTimestamp( pcr );
-            AbstractCommand *command = commandsqueue->factory()->createEsOutControlPCRCommand( i_group, pcr );
+
+            SegmentTimes times;
+
+            if(synchronizationReference.second.segment.demux != VLC_TS_INVALID)
+            {
+                pcr = fixTimestamp( pcr );
+
+                pcr = applyTimestampContinuity( pcr );
+
+                times = synchronizationReference.second.segment;
+                times.offsetBy(pcr - times.demux);
+            }
+            else pcr = VLC_TS_INVALID;
+
+            AbstractCommand *command = commandsfactory->createEsOutControlPCRCommand( i_group, times, pcr );
             if( likely(command) )
             {
                 commandsqueue->Schedule( command );
@@ -542,7 +696,7 @@ int FakeESOut::esOutControl(int i_query, va_list args)
         {
             static_cast<void>(va_arg( args, int )); /* ignore group */
             const vlc_meta_t *p_meta = va_arg( args, const vlc_meta_t * );
-            AbstractCommand *command = commandsqueue->factory()->createEsOutMetaCommand( -1, p_meta );
+            AbstractCommand *command = commandsfactory->createEsOutMetaCommand( this,-1, p_meta );
             if( likely(command) )
             {
                 commandsqueue->Schedule( command );
@@ -573,7 +727,9 @@ void FakeESOut::esOutDestroy()
 {
     vlc_mutex_locker locker(&lock);
 
-    AbstractCommand *command = commandsqueue->factory()->createEsOutDestroyCommand();
+    scheduleNecessaryMilestone();
+
+    AbstractCommand *command = commandsfactory->createEsOutDestroyCommand();
     if( likely(command) )
         commandsqueue->Schedule( command );
 }
