@@ -2,7 +2,7 @@
  * ps.c: Program Stream demux module for VLC.
  *****************************************************************************
  * Copyright (C) 2004-2009 VLC authors and VideoLAN
- * $Id: 91465f361646a520e372233cffb552dd1b3be889 $
+ * $Id: 71a94ec8e0a472f57fc4ac5632b49fcb9dffd688 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -46,6 +46,11 @@
     "to calculate position and duration. However sometimes this might not " \
     "be usable. Disable this option to calculate from the bitrate instead." )
 
+#define PS_PACKET_PROBE 3
+#define CDXA_HEADER_SIZE 44
+#define CDXA_SECTOR_SIZE 2352
+#define CDXA_SECTOR_HEADER_SIZE 24
+
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
@@ -82,25 +87,36 @@ struct demux_sys_t
     ps_psm_t    psm;
     ps_track_t  tk[PS_TK_COUNT];
 
-    int64_t     i_scr;
-    int64_t     i_last_scr;
+    int64_t     i_pack_scr; /* current read pack scr value, temp */
+    int64_t     i_first_scr; /* media offset */
+    int64_t     i_scr; /* committed, current position */
+    int64_t     i_scr_track_id;
     int         i_mux_rate;
     int64_t     i_length;
-    int         i_time_track;
+    int         i_time_track_index;
     int64_t     i_current_pts;
+    uint64_t    i_start_byte;
+    uint64_t    i_lastpack_byte;
 
     int         i_aob_mlp_count;
 
     bool  b_lost_sync;
     bool  b_have_pack;
+    bool  b_bad_scr;
     bool  b_seekable;
+    enum
+    {
+        MPEG_PS = 0,
+        CDXA_PS,
+        PSMF_PS,
+    } format;
 };
 
 static int Demux  ( demux_t *p_demux );
 static int Control( demux_t *p_demux, int i_query, va_list args );
 
-static int      ps_pkt_resynch( stream_t *, uint32_t *pi_code );
-static block_t *ps_pkt_read   ( stream_t *, uint32_t i_code );
+static int      ps_pkt_resynch( stream_t *, int, bool );
+static block_t *ps_pkt_read   ( stream_t * );
 
 /*****************************************************************************
  * Open
@@ -111,22 +127,77 @@ static int OpenCommon( vlc_object_t *p_this, bool b_force )
     demux_sys_t *p_sys;
 
     const uint8_t *p_peek;
+    ssize_t i_peek = 0;
+    ssize_t i_offset = 0;
+    ssize_t i_skip = 0;
+    unsigned i_max_packets = PS_PACKET_PROBE;
+    int format = MPEG_PS;
+    int i_mux_rate = 0;
+    int i_length = -1;
 
-    if( stream_Peek( p_demux->s, &p_peek, 4 ) < 4 )
+    i_peek = vlc_stream_Peek( p_demux->s, &p_peek, 16 );
+    if( i_peek < 16 )
     {
-        msg_Err( p_demux, "cannot peek" );
+        msg_Dbg( p_demux, "cannot peek" );
         return VLC_EGENERIC;
     }
 
-    if( memcmp( p_peek, "\x00\x00\x01", 3 ) || ( p_peek[3] < 0xb9 ) )
+    if( !memcmp( p_peek, "PSMF", 4 ) &&
+        (GetDWBE( &p_peek[4] ) & 0x30303030) == 0x30303030 )
     {
-        if( !b_force )
+        i_peek = vlc_stream_Peek( p_demux->s, &p_peek, 100 );
+        if( i_peek < 100 )
             return VLC_EGENERIC;
-
+        i_skip = i_offset = GetWBE( &p_peek[10] );
+        format = PSMF_PS;
+        msg_Info( p_demux, "Detected PSMF-PS header");
+        i_mux_rate = GetDWBE( &p_peek[96] );
+        if( GetDWBE( &p_peek[86] ) > 0 )
+            i_length = CLOCK_FREQ * GetDWBE( &p_peek[92] ) / GetDWBE( &p_peek[86] );
+    }
+    else if( !memcmp( p_peek, "RIFF", 4 ) && !memcmp( &p_peek[8], "CDXA", 4 ) )
+    {
+        format = CDXA_PS;
+        i_max_packets = 0; /* We can't probe here */
+        i_skip = CDXA_HEADER_SIZE;
+        msg_Info( p_demux, "Detected CDXA-PS" );
+        /* FIXME: have a proper way to decap CD sectors or make an access stream filter */
+    }
+    else if( b_force )
+    {
         msg_Warn( p_demux, "this does not look like an MPEG PS stream, "
                   "continuing anyway" );
+        i_max_packets = 0;
     }
 
+    for( unsigned i=0; i<i_max_packets; i++ )
+    {
+        if( i_peek < i_offset + 16 )
+        {
+            i_peek = vlc_stream_Peek( p_demux->s, &p_peek, i_offset + 16 );
+            if( i_peek < i_offset + 16 )
+                return VLC_EGENERIC;
+        }
+
+        const uint8_t startcode[3] = { 0x00, 0x00, 0x01 };
+        const uint8_t *p_header = &p_peek[i_offset];
+        if( memcmp( p_header, startcode, 3 ) ||
+           ( (p_header[3] & 0xB0) != 0xB0 &&
+            !(p_header[3] >= 0xC0 && p_header[3] <= 0xEF) &&
+              p_header[3] != PS_STREAM_ID_EXTENDED &&
+              p_header[3] != PS_STREAM_ID_DIRECTORY ) )
+            return VLC_EGENERIC;
+
+        ssize_t i_pessize = ps_pkt_size( p_header, 16 );
+        if( i_pessize < 5 )
+            return VLC_EGENERIC;
+        i_offset += i_pessize;
+    }
+
+    if( i_skip > 0 && !p_demux->b_preparsing &&
+        vlc_stream_Read( p_demux->s, NULL, i_skip ) != i_skip )
+        return VLC_EGENERIC;
+
     /* Fill p_demux field */
     p_demux->p_sys = p_sys = malloc( sizeof( demux_sys_t ) );
     if( !p_sys ) return VLC_ENOMEM;
@@ -135,19 +206,25 @@ static int OpenCommon( vlc_object_t *p_this, bool b_force )
     p_demux->pf_control = Control;
 
     /* Init p_sys */
-    p_sys->i_mux_rate = 0;
-    p_sys->i_scr      = -1;
-    p_sys->i_last_scr = -1;
-    p_sys->i_length   = -1;
+    p_sys->i_mux_rate = i_mux_rate;
+    p_sys->i_pack_scr      = -1;
+    p_sys->i_first_scr = -1;
+    p_sys->i_scr = -1;
+    p_sys->i_scr_track_id = 0;
+    p_sys->i_length   = i_length;
     p_sys->i_current_pts = (mtime_t) 0;
-    p_sys->i_time_track = -1;
+    p_sys->i_time_track_index = -1;
     p_sys->i_aob_mlp_count = 0;
+    p_sys->i_start_byte = i_skip;
+    p_sys->i_lastpack_byte = i_skip;
 
     p_sys->b_lost_sync = false;
     p_sys->b_have_pack = false;
+    p_sys->b_bad_scr   = false;
     p_sys->b_seekable  = false;
+    p_sys->format      = format;
 
-    stream_Control( p_demux->s, STREAM_CAN_SEEK, &p_sys->b_seekable );
+    vlc_stream_Control( p_demux->s, STREAM_CAN_SEEK, &p_sys->b_seekable );
 
     ps_psm_init( &p_sys->psm );
     ps_track_init( p_sys->tk );
@@ -164,7 +241,7 @@ static int OpenForce( vlc_object_t *p_this )
 
 static int Open( vlc_object_t *p_this )
 {
-    return OpenCommon( p_this, ((demux_t *)p_this)->b_force );
+    return OpenCommon( p_this, p_this->obj.force );
 }
 
 /*****************************************************************************
@@ -179,7 +256,7 @@ static void Close( vlc_object_t *p_this )
     for( i = 0; i < PS_TK_COUNT; i++ )
     {
         ps_track_t *tk = &p_sys->tk[i];
-        if( tk->b_seen )
+        if( tk->b_configured )
         {
             es_format_Clean( &tk->fmt );
             if( tk->es ) es_out_Del( p_demux->out, tk->es );
@@ -191,17 +268,16 @@ static void Close( vlc_object_t *p_this )
     free( p_sys );
 }
 
-static int Demux2( demux_t *p_demux, bool b_end )
+static int Probe( demux_t *p_demux, bool b_end )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     int i_ret, i_id;
-    uint32_t i_code;
     block_t *p_pkt;
 
-    i_ret = ps_pkt_resynch( p_demux->s, &i_code );
+    i_ret = ps_pkt_resynch( p_demux->s, p_sys->format, p_sys->b_have_pack );
     if( i_ret < 0 )
     {
-        return 0;
+        return VLC_DEMUXER_EOF;
     }
     else if( i_ret == 0 )
     {
@@ -209,19 +285,21 @@ static int Demux2( demux_t *p_demux, bool b_end )
             msg_Warn( p_demux, "garbage at input, trying to resync..." );
 
         p_sys->b_lost_sync = true;
-        return 1;
+        return VLC_DEMUXER_SUCCESS;
     }
 
     if( p_sys->b_lost_sync ) msg_Warn( p_demux, "found sync code" );
     p_sys->b_lost_sync = false;
 
-    if( ( p_pkt = ps_pkt_read( p_demux->s, i_code ) ) == NULL )
+    if( ( p_pkt = ps_pkt_read( p_demux->s ) ) == NULL )
     {
-        return 0;
+        return VLC_DEMUXER_EOF;
     }
-    if( (i_id = ps_pkt_id( p_pkt )) >= 0xc0 )
+
+    i_id = ps_pkt_id( p_pkt );
+    if( i_id >= 0xc0 )
     {
-        ps_track_t *tk = &p_sys->tk[PS_ID_TO_TK(i_id)];
+        ps_track_t *tk = &p_sys->tk[ps_id_to_tk(i_id)];
         if( !ps_pkt_parse_pes( VLC_OBJECT(p_demux), p_pkt, tk->i_skip ) &&
              p_pkt->i_pts > VLC_TS_INVALID )
         {
@@ -235,52 +313,90 @@ static int Demux2( demux_t *p_demux, bool b_end )
             }
         }
     }
+    else if( i_id == PS_STREAM_ID_PACK_HEADER )
+    {
+        int64_t i_scr; int dummy;
+        if( !b_end && !ps_pkt_parse_pack( p_pkt, &i_scr, &dummy ) )
+        {
+            if( p_sys->i_first_scr == -1 )
+                p_sys->i_first_scr = i_scr;
+        }
+        p_sys->b_have_pack = true;
+    }
+
     block_Release( p_pkt );
-    return 1;
+    return VLC_DEMUXER_SUCCESS;
 }
 
-static void FindLength( demux_t *p_demux )
+static bool FindLength( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     int64_t i_current_pos = -1, i_size = 0, i_end = 0;
 
     if( !var_CreateGetBool( p_demux, "ps-trust-timestamps" ) )
-        return;
+        return true;
 
     if( p_sys->i_length == -1 ) /* First time */
     {
         p_sys->i_length = 0;
         /* Check beginning */
         int i = 0;
-        i_current_pos = stream_Tell( p_demux->s );
-        while( i < 40 && Demux2( p_demux, false ) > 0 ) i++;
+        i_current_pos = vlc_stream_Tell( p_demux->s );
+        while( i < 40 && Probe( p_demux, false ) > 0 ) i++;
 
         /* Check end */
         i_size = stream_Size( p_demux->s );
         i_end = VLC_CLIP( i_size, 0, 200000 );
-        stream_Seek( p_demux->s, i_size - i_end );
-
-        i = 0;
-        while( i < 400 && Demux2( p_demux, true ) > 0 ) i++;
-        if( i_current_pos >= 0 ) stream_Seek( p_demux->s, i_current_pos );
+        if( vlc_stream_Seek( p_demux->s, i_size - i_end ) == VLC_SUCCESS )
+        {
+            i = 0;
+            while( i < 400 && Probe( p_demux, true ) > 0 ) i++;
+            if( i_current_pos >= 0 &&
+                vlc_stream_Seek( p_demux->s, i_current_pos ) != VLC_SUCCESS )
+                    return false;
+        }
+        else return false;
     }
 
     /* Find the longest track */
     for( int i = 0; i < PS_TK_COUNT; i++ )
     {
         ps_track_t *tk = &p_sys->tk[i];
-        if( tk->i_last_pts > 0 &&
+        if( tk->i_first_pts >= 0 &&
             tk->i_last_pts > tk->i_first_pts )
         {
             int64_t i_length = (int64_t)tk->i_last_pts - tk->i_first_pts;
             if( i_length > p_sys->i_length )
             {
                 p_sys->i_length = i_length;
-                p_sys->i_time_track = i;
+                p_sys->i_time_track_index = i;
                 msg_Dbg( p_demux, "we found a length of: %"PRId64 "s", p_sys->i_length / CLOCK_FREQ );
             }
         }
     }
+    return true;
+}
+
+static void NotifyDiscontinuity( ps_track_t *p_tk, es_out_t *out )
+{
+    bool b_selected;
+    for( size_t i = 0; i < PS_TK_COUNT; i++ )
+    {
+        ps_track_t *tk = &p_tk[i];
+        if( tk->es &&
+                es_out_Control( out, ES_OUT_GET_ES_STATE, tk->es, &b_selected ) == VLC_SUCCESS
+                && b_selected )
+        {
+            tk->i_next_block_flags |= BLOCK_FLAG_DISCONTINUITY;
+        }
+    }
+}
+
+static void CheckPCR( demux_sys_t *p_sys, es_out_t *out, mtime_t i_scr )
+{
+    if( p_sys->i_scr > VLC_TS_INVALID &&
+        llabs( p_sys->i_scr - i_scr ) > CLOCK_FREQ )
+        NotifyDiscontinuity( p_sys->tk, out );
 }
 
 /*****************************************************************************
@@ -289,54 +405,71 @@ static void FindLength( demux_t *p_demux )
 static int Demux( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
-    int i_ret, i_id, i_mux_rate;
-    uint32_t i_code;
+    int i_ret, i_mux_rate;
     block_t *p_pkt;
 
-    i_ret = ps_pkt_resynch( p_demux->s, &i_code );
+    i_ret = ps_pkt_resynch( p_demux->s, p_sys->format, p_sys->b_have_pack );
     if( i_ret < 0 )
     {
-        return 0;
+        return VLC_DEMUXER_EOF;
     }
     else if( i_ret == 0 )
     {
         if( !p_sys->b_lost_sync )
-            msg_Warn( p_demux, "garbage at input, trying to resync..." );
+        {
+            msg_Warn( p_demux, "garbage at input from %"PRIu64", trying to resync...",
+                                vlc_stream_Tell(p_demux->s) );
+            NotifyDiscontinuity( p_sys->tk, p_demux->out );
+        }
 
         p_sys->b_lost_sync = true;
-        return 1;
+        return VLC_DEMUXER_SUCCESS;
     }
 
     if( p_sys->b_lost_sync ) msg_Warn( p_demux, "found sync code" );
     p_sys->b_lost_sync = false;
 
     if( p_sys->i_length < 0 && p_sys->b_seekable )
-        FindLength( p_demux );
-
-    if( ( p_pkt = ps_pkt_read( p_demux->s, i_code ) ) == NULL )
     {
-        return 0;
+        if( !FindLength( p_demux ) )
+            return VLC_DEMUXER_EGENERIC;
     }
 
-    switch( i_code )
+    if( ( p_pkt = ps_pkt_read( p_demux->s ) ) == NULL )
     {
-    case 0x1b9:
+        return VLC_DEMUXER_EOF;
+    }
+
+    if( p_pkt->i_buffer < 4 )
+    {
+        block_Release( p_pkt );
+        return VLC_DEMUXER_EGENERIC;
+    }
+
+    const uint8_t i_stream_id = p_pkt->p_buffer[3];
+    switch( i_stream_id )
+    {
+    case PS_STREAM_ID_END_STREAM:
         block_Release( p_pkt );
         break;
 
-    case 0x1ba:
-        if( !ps_pkt_parse_pack( p_pkt, &p_sys->i_scr, &i_mux_rate ) )
+    case PS_STREAM_ID_PACK_HEADER:
+        if( !ps_pkt_parse_pack( p_pkt, &p_sys->i_pack_scr, &i_mux_rate ) )
         {
-            p_sys->i_last_scr = p_sys->i_scr;
+            if( p_sys->i_first_scr == -1 )
+                p_sys->i_first_scr = p_sys->i_pack_scr;
+            CheckPCR( p_sys, p_demux->out, p_sys->i_pack_scr );
+            p_sys->i_scr = p_sys->i_pack_scr;
+            p_sys->i_lastpack_byte = vlc_stream_Tell( p_demux->s );
             if( !p_sys->b_have_pack ) p_sys->b_have_pack = true;
             /* done later on to work around bad vcd/svcd streams */
-            /* es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_sys->i_scr ); */
+            /* es_out_SetPCR( p_demux->out, p_sys->i_scr ); */
             if( i_mux_rate > 0 ) p_sys->i_mux_rate = i_mux_rate;
         }
         block_Release( p_pkt );
         break;
 
-    case 0x1bb:
+    case PS_STREAM_ID_SYSTEM_HEADER:
         if( !ps_pkt_parse_system( p_pkt, &p_sys->psm, p_sys->tk ) )
         {
             int i;
@@ -344,16 +477,19 @@ static int Demux( demux_t *p_demux )
             {
                 ps_track_t *tk = &p_sys->tk[i];
 
-                if( tk->b_seen && !tk->es && tk->fmt.i_cat != UNKNOWN_ES )
+                if( !tk->b_configured && tk->fmt.i_cat != UNKNOWN_ES )
                 {
-                    tk->es = es_out_Add( p_demux->out, &tk->fmt );
+                    if( tk->b_seen )
+                        tk->es = es_out_Add( p_demux->out, &tk->fmt );
+                     /* else create when seeing packet */
+                    tk->b_configured = true;
                 }
             }
         }
         block_Release( p_pkt );
         break;
 
-    case 0x1bc:
+    case PS_STREAM_ID_MAP:
         if( p_sys->psm.i_version == 0xFFFF )
             msg_Dbg( p_demux, "contains a PSM");
 
@@ -362,8 +498,17 @@ static int Demux( demux_t *p_demux )
         break;
 
     default:
-        if( (i_id = ps_pkt_id( p_pkt )) >= 0xc0 )
+        /* Reject non video/audio nor PES */
+        if( i_stream_id < 0xC0 || i_stream_id > 0xEF )
         {
+            block_Release( p_pkt );
+            break;
+        }
+        /* fallthrough */
+    case PS_STREAM_ID_PRIVATE_STREAM1:
+    case PS_STREAM_ID_EXTENDED:
+        {
+            int i_id = ps_pkt_id( p_pkt );
             /* Small heuristic to improve MLP detection from AOB */
             if( i_id == 0xa001 &&
                 p_sys->i_aob_mlp_count < 500 )
@@ -378,54 +523,107 @@ static int Demux( demux_t *p_demux )
             }
 
             bool b_new = false;
-            ps_track_t *tk = &p_sys->tk[PS_ID_TO_TK(i_id)];
+            ps_track_t *tk = &p_sys->tk[ps_id_to_tk(i_id)];
 
-            if( !tk->b_seen )
+            if( !tk->b_configured )
             {
-                if( !ps_track_fill( tk, &p_sys->psm, i_id, p_pkt ) )
+                if( !ps_track_fill( tk, &p_sys->psm, i_id, p_pkt, false ) )
                 {
+                    /* No PSM and no probing yet */
+                    if( p_sys->format == PSMF_PS )
+                    {
+                        if( tk->fmt.i_cat == VIDEO_ES )
+                            tk->fmt.i_codec = VLC_CODEC_H264;
+#if 0
+                        if( i_stream_id == PS_STREAM_ID_PRIVATE_STREAM1 )
+                        {
+                            es_format_Change( &tk->fmt, AUDIO_ES, VLC_CODEC_ATRAC3P );
+                            tk->fmt.audio.i_blockalign = 376;
+                            tk->fmt.audio.i_channels = 2;
+                            tk->fmt.audio.i_rate = 44100;
+                        }
+#endif
+                    }
+
                     tk->es = es_out_Add( p_demux->out, &tk->fmt );
                     b_new = true;
+                    tk->b_configured = true;
                 }
                 else
                 {
                     msg_Dbg( p_demux, "es id=0x%x format unknown", i_id );
                 }
-                tk->b_seen = true;
             }
 
+            /* Late creation from system header */
+            if( !tk->b_seen && tk->b_configured && !tk->es && tk->fmt.i_cat != UNKNOWN_ES )
+                tk->es = es_out_Add( p_demux->out, &tk->fmt );
+
+            tk->b_seen = true;
+
             /* The popular VCD/SVCD subtitling WinSubMux does not
              * renumber the SCRs when merging subtitles into the PES */
-            if( tk->b_seen &&
+            if( tk->b_seen && !p_sys->b_bad_scr &&
                 ( tk->fmt.i_codec == VLC_CODEC_OGT ||
                   tk->fmt.i_codec == VLC_CODEC_CVD ) )
             {
-                p_sys->i_scr = -1;
-                p_sys->i_last_scr = -1;
+                p_sys->b_bad_scr = true;
+                p_sys->i_first_scr = -1;
             }
 
-            if( p_sys->i_scr >= 0 )
-                es_out_Control( p_demux->out, ES_OUT_SET_PCR, VLC_TS_0 + p_sys->i_scr );
+            if( p_sys->i_pack_scr >= 0 && !p_sys->b_bad_scr )
+            {
+                if( (tk->fmt.i_cat == AUDIO_ES || tk->fmt.i_cat == VIDEO_ES) &&
+                    tk->i_first_pts > VLC_TS_INVALID && tk->i_first_pts - p_sys->i_pack_scr > 2 * CLOCK_FREQ )
+                {
+                    msg_Warn( p_demux, "Incorrect SCR timing offset by of %"PRId64 "ms, disabling",
+                                       tk->i_first_pts - p_sys->i_pack_scr / 1000 );
+                    p_sys->b_bad_scr = true; /* Disable Offset SCR */
+                    p_sys->i_first_scr = -1;
+                }
+                else
+                    es_out_SetPCR( p_demux->out, VLC_TS_0 + p_sys->i_pack_scr );
+            }
 
-            p_sys->i_scr = -1;
-
-            if( tk->b_seen && tk->es &&
+            if( tk->b_configured && tk->es &&
                 !ps_pkt_parse_pes( VLC_OBJECT(p_demux), p_pkt, tk->i_skip ) )
             {
-                if( !b_new && !p_sys->b_have_pack &&
-                    (tk->fmt.i_cat == AUDIO_ES) &&
-                    (p_pkt->i_pts > VLC_TS_INVALID) )
+                if( tk->fmt.i_cat == AUDIO_ES || tk->fmt.i_cat == VIDEO_ES )
+                {
+                    if( !p_sys->b_bad_scr && p_sys->i_pack_scr > 0 && p_pkt->i_pts > 0 &&
+                        p_sys->i_pack_scr > p_pkt->i_pts + CLOCK_FREQ / 4 )
+                    {
+                        msg_Warn( p_demux, "Incorrect SCR timing in advance of %" PRId64 "ms, disabling",
+                                           p_sys->i_pack_scr - p_pkt->i_pts / 1000 );
+                        p_sys->b_bad_scr = true;
+                        p_sys->i_first_scr = -1;
+                    }
+
+                    if( (p_sys->b_bad_scr || !p_sys->b_have_pack) && !p_sys->i_scr_track_id )
+                    {
+                        p_sys->i_scr_track_id = tk->i_id;
+                    }
+                }
+
+                if( ((!b_new && !p_sys->b_have_pack) || p_sys->b_bad_scr) &&
+                    p_sys->i_scr_track_id == tk->i_id &&
+                    p_pkt->i_pts > VLC_TS_INVALID )
                 {
                     /* A hack to sync the A/V on PES files. */
                     msg_Dbg( p_demux, "force SCR: %"PRId64, p_pkt->i_pts );
-                    es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_pkt->i_pts );
+                    CheckPCR( p_sys, p_demux->out, p_pkt->i_pts );
+                    p_sys->i_scr = p_pkt->i_pts;
+                    if( p_sys->i_first_scr == -1 )
+                        p_sys->i_first_scr = p_sys->i_scr;
+                    es_out_SetPCR( p_demux->out, p_pkt->i_pts );
                 }
+
                 if( tk->fmt.i_codec == VLC_CODEC_TELETEXT &&
-                    p_pkt->i_pts <= VLC_TS_INVALID && p_sys->i_last_scr >= 0 )
+                    p_pkt->i_pts <= VLC_TS_INVALID && p_sys->i_scr >= 0 )
                 {
                     /* Teletext may have missing PTS (ETSI EN 300 472 Annexe A)
                      * In this case use the last SCR + 40ms */
-                    p_pkt->i_pts = VLC_TS_0 + p_sys->i_last_scr + 40000;
+                    p_pkt->i_pts = VLC_TS_0 + p_sys->i_scr + 40000;
                 }
 
                 if( (int64_t)p_pkt->i_pts > p_sys->i_current_pts )
@@ -433,22 +631,32 @@ static int Demux( demux_t *p_demux )
                     p_sys->i_current_pts = (int64_t)p_pkt->i_pts;
                 }
 
+                if( tk->i_next_block_flags )
+                {
+                    p_pkt->i_flags = tk->i_next_block_flags;
+                    tk->i_next_block_flags = 0;
+                }
+#if 0
+                if( tk->fmt.i_codec == VLC_CODEC_ATRAC3P )
+                {
+                    p_pkt->p_buffer += 14;
+                    p_pkt->i_buffer -= 14;
+                }
+#endif
                 es_out_Send( p_demux->out, tk->es, p_pkt );
             }
             else
             {
                 block_Release( p_pkt );
             }
-        }
-        else
-        {
-            block_Release( p_pkt );
+
+            p_sys->i_pack_scr = -1;
         }
         break;
     }
 
     demux_UpdateTitleFromStream( p_demux );
-    return 1;
+    return VLC_DEMUXER_SUCCESS;
 }
 
 /*****************************************************************************
@@ -459,6 +667,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
     demux_sys_t *p_sys = p_demux->p_sys;
     double f, *pf;
     int64_t i64, *pi64;
+    int i_ret;
 
     switch( i_query )
     {
@@ -467,11 +676,11 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             return VLC_SUCCESS;
 
         case DEMUX_GET_POSITION:
-            pf = (double*) va_arg( args, double* );
-            i64 = stream_Size( p_demux->s );
+            pf = va_arg( args, double * );
+            i64 = stream_Size( p_demux->s ) - p_sys->i_start_byte;
             if( i64 > 0 )
             {
-                double current = stream_Tell( p_demux->s );
+                double current = vlc_stream_Tell( p_demux->s ) - p_sys->i_start_byte;
                 *pf = current / (double)i64;
             }
             else
@@ -481,31 +690,52 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             return VLC_SUCCESS;
 
         case DEMUX_SET_POSITION:
-            f = (double) va_arg( args, double );
-            i64 = stream_Size( p_demux->s );
+            f = va_arg( args, double );
+            i64 = stream_Size( p_demux->s ) - p_sys->i_start_byte;
             p_sys->i_current_pts = 0;
-            p_sys->i_last_scr = -1;
+            p_sys->i_scr = -1;
 
-            return stream_Seek( p_demux->s, (int64_t)(i64 * f) );
+            if( p_sys->format == CDXA_PS )
+            {
+                i64 = (int64_t)(i64  * f); /* Align to sector payload */
+                i64 = p_sys->i_start_byte + i64 - (i64 % CDXA_SECTOR_SIZE) + CDXA_SECTOR_HEADER_SIZE;
+            }
+            else
+            {
+                i64 = p_sys->i_start_byte + (int64_t)(i64 * f);
+            }
+
+            i_ret = vlc_stream_Seek( p_demux->s, i64 );
+            if( i_ret == VLC_SUCCESS )
+            {
+                NotifyDiscontinuity( p_sys->tk, p_demux->out );
+                return i_ret;
+            }
+            break;
 
         case DEMUX_GET_TIME:
-            pi64 = (int64_t*)va_arg( args, int64_t * );
-            if( p_sys->i_time_track >= 0 && p_sys->i_current_pts > 0 )
+            pi64 = va_arg( args, int64_t * );
+            if( p_sys->i_time_track_index >= 0 && p_sys->i_current_pts > 0 )
             {
-                *pi64 = p_sys->i_current_pts - p_sys->tk[p_sys->i_time_track].i_first_pts;
+                *pi64 = p_sys->i_current_pts - p_sys->tk[p_sys->i_time_track_index].i_first_pts;
                 return VLC_SUCCESS;
             }
-            if( p_sys->i_mux_rate > 0 )
+            if( p_sys->i_first_scr > -1 && p_sys->i_scr > -1 )
             {
-                *pi64 = (int64_t)1000000 * ( stream_Tell( p_demux->s ) / 50 ) /
-                    p_sys->i_mux_rate;
+                *pi64 = p_sys->i_scr - p_sys->i_first_scr;
+                /* H.222 2.5.2.2 */
+                if( p_sys->i_mux_rate > 0 && p_sys->b_have_pack )
+                {
+                    uint64_t i_offset = vlc_stream_Tell( p_demux->s ) - p_sys->i_lastpack_byte;
+                    *pi64 += CLOCK_FREQ * i_offset / (p_sys->i_mux_rate * 50);
+                }
                 return VLC_SUCCESS;
             }
             *pi64 = 0;
-            return VLC_EGENERIC;
+            break;
 
         case DEMUX_GET_LENGTH:
-            pi64 = (int64_t*)va_arg( args, int64_t * );
+            pi64 = va_arg( args, int64_t * );
             if( p_sys->i_length > 0 )
             {
                 *pi64 = p_sys->i_length;
@@ -513,30 +743,21 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             }
             else if( p_sys->i_mux_rate > 0 )
             {
-                *pi64 = (int64_t)1000000 * ( stream_Size( p_demux->s ) / 50 ) /
+                *pi64 = CLOCK_FREQ * ( stream_Size( p_demux->s ) - p_sys->i_start_byte / 50 ) /
                     p_sys->i_mux_rate;
                 return VLC_SUCCESS;
             }
             *pi64 = 0;
-            return VLC_EGENERIC;
+            break;
 
         case DEMUX_SET_TIME:
-            i64 = (int64_t)va_arg( args, int64_t );
-            if( p_sys->i_time_track >= 0 && p_sys->i_current_pts > 0 )
+            i64 = va_arg( args, int64_t );
+            if( p_sys->i_time_track_index >= 0 && p_sys->i_current_pts > 0 && p_sys->i_length )
             {
-                int64_t i_now = p_sys->i_current_pts - p_sys->tk[p_sys->i_time_track].i_first_pts;
-                int64_t i_pos = stream_Tell( p_demux->s );
-
-                if( !i_now )
-                    return i64 ? VLC_EGENERIC : VLC_SUCCESS;
-
-                p_sys->i_current_pts = 0;
-                p_sys->i_last_scr = -1;
-                i_pos *= (float)i64 / (float)i_now;
-                stream_Seek( p_demux->s, i_pos );
-                return VLC_SUCCESS;
+                i64 -= p_sys->tk[p_sys->i_time_track_index].i_first_pts;
+                return demux_Control( p_demux, DEMUX_SET_POSITION, (double) i64 / p_sys->i_length );
             }
-            return VLC_EGENERIC;
+            break;
 
         case DEMUX_GET_TITLE_INFO:
         {
@@ -545,22 +766,25 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
 
             *va_arg( args, int* ) = 0; /* Title offset */
             *va_arg( args, int* ) = 0; /* Chapter offset */
-            return stream_Control( p_demux->s, STREAM_GET_TITLE_INFO, v, c );
+            return vlc_stream_Control( p_demux->s, STREAM_GET_TITLE_INFO, v,
+                                       c );
         }
 
         case DEMUX_SET_TITLE:
-            return stream_vaControl( p_demux->s, STREAM_SET_TITLE, args );
+            return vlc_stream_vaControl( p_demux->s, STREAM_SET_TITLE, args );
 
         case DEMUX_SET_SEEKPOINT:
-            return stream_vaControl( p_demux->s, STREAM_SET_SEEKPOINT, args );
+            return vlc_stream_vaControl( p_demux->s, STREAM_SET_SEEKPOINT,
+                                         args );
 
         case DEMUX_GET_META:
-            return stream_vaControl( p_demux->s, STREAM_GET_META, args );
+            return vlc_stream_vaControl( p_demux->s, STREAM_GET_META, args );
 
         case DEMUX_GET_FPS:
         default:
-            return VLC_EGENERIC;
+            break;
     }
+    return VLC_EGENERIC;
 }
 
 /*****************************************************************************
@@ -571,24 +795,23 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
  *  It doesn't skip more than 512 bytes
  *  -1 -> error, 0 -> not synch, 1 -> ok
  */
-static int ps_pkt_resynch( stream_t *s, uint32_t *pi_code )
+static int ps_pkt_resynch( stream_t *s, int format, bool b_pack )
 {
     const uint8_t *p_peek;
     int     i_peek;
     int     i_skip;
 
-    if( stream_Peek( s, &p_peek, 4 ) < 4 )
+    if( vlc_stream_Peek( s, &p_peek, 4 ) < 4 )
     {
         return -1;
     }
     if( p_peek[0] == 0 && p_peek[1] == 0 && p_peek[2] == 1 &&
-        p_peek[3] >= 0xb9 )
+        p_peek[3] >= PS_STREAM_ID_END_STREAM )
     {
-        *pi_code = 0x100 | p_peek[3];
         return 1;
     }
 
-    if( ( i_peek = stream_Peek( s, &p_peek, 512 ) ) < 4 )
+    if( ( i_peek = vlc_stream_Peek( s, &p_peek, 512 ) ) < 4 )
     {
         return -1;
     }
@@ -600,35 +823,50 @@ static int ps_pkt_resynch( stream_t *s, uint32_t *pi_code )
         {
             break;
         }
-        if( p_peek[0] == 0 && p_peek[1] == 0 && p_peek[2] == 1 &&
-            p_peek[3] >= 0xb9 )
+        /* Handle mid stream 24 bytes padding+CRC creating emulated sync codes with incorrect
+           PES sizes and frelling up to UINT16_MAX bytes followed by 24 bytes CDXA Header */
+        if( format == CDXA_PS && i_skip == 0 && i_peek >= 48 )
         {
-            *pi_code = 0x100 | p_peek[3];
-            return stream_Read( s, NULL, i_skip ) == i_skip ? 1 : -1;
+            const uint8_t cdxasynccode[12] = { 0x00, 0xff, 0xff, 0xff, 0xff, 0xff,
+                                               0xff, 0xff, 0xff, 0xff, 0xff, 0x00 };
+            if( !memcmp( &p_peek[24], cdxasynccode, 12 ) )
+            {
+                i_peek -= 48;
+                p_peek += 48;
+                i_skip += 48;
+                continue;
+            }
+        }
+
+        if( p_peek[0] == 0 && p_peek[1] == 0 && p_peek[2] == 1 &&
+            p_peek[3] >= PS_STREAM_ID_END_STREAM &&
+            ( !b_pack || p_peek[3] == PS_STREAM_ID_PACK_HEADER ) )
+        {
+            return vlc_stream_Read( s, NULL, i_skip ) == i_skip ? 1 : -1;
         }
 
         p_peek++;
         i_peek--;
         i_skip++;
     }
-    return stream_Read( s, NULL, i_skip ) == i_skip ? 0 : -1;
+    return vlc_stream_Read( s, NULL, i_skip ) == i_skip ? 0 : -1;
 }
 
-static block_t *ps_pkt_read( stream_t *s, uint32_t i_code )
+static block_t *ps_pkt_read( stream_t *s )
 {
     const uint8_t *p_peek;
-    int i_peek = stream_Peek( s, &p_peek, 14 );
+    int i_peek = vlc_stream_Peek( s, &p_peek, 14 );
     if( i_peek < 4 )
         return NULL;
 
     int i_size = ps_pkt_size( p_peek, i_peek );
-    if( i_size <= 6 && p_peek[3] > 0xba )
+    if( i_size <= 6 && p_peek[3] > PS_STREAM_ID_PACK_HEADER )
     {
         /* Special case, search the next start code */
         i_size = 6;
         for( ;; )
         {
-            i_peek = stream_Peek( s, &p_peek, i_size + 1024 );
+            i_peek = vlc_stream_Peek( s, &p_peek, i_size + 1024 );
             if( i_peek <= i_size + 4 )
             {
                 return NULL;
@@ -636,9 +874,9 @@ static block_t *ps_pkt_read( stream_t *s, uint32_t i_code )
             while( i_size <= i_peek - 4 )
             {
                 if( p_peek[i_size] == 0x00 && p_peek[i_size+1] == 0x00 &&
-                    p_peek[i_size+2] == 0x01 && p_peek[i_size+3] >= 0xb9 )
+                    p_peek[i_size+2] == 0x01 && p_peek[i_size+3] >= PS_STREAM_ID_END_STREAM )
                 {
-                    return stream_Block( s, i_size );
+                    return vlc_stream_Block( s, i_size );
                 }
                 i_size++;
             }
@@ -647,9 +885,8 @@ static block_t *ps_pkt_read( stream_t *s, uint32_t i_code )
     else
     {
         /* Normal case */
-        return stream_Block( s, i_size );
+        return vlc_stream_Block( s, i_size );
     }
 
-    VLC_UNUSED(i_code);
     return NULL;
 }
