@@ -6,7 +6,7 @@
  * based on code by Christopher Wingert for tivo-mplayer
  * tivo(at)wingert.org, February 2003
  *
- * $Id: 5507fa6bcf8994c7aea3e74140247ba77544ab36 $
+ * $Id: d96ea139bf8ac0ae940c7c63ea49ca09b81fd084 $
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -166,7 +166,7 @@ typedef enum
 typedef struct
 {
     bool b_started;
-    int        i_data;
+    size_t     i_data;
     uint8_t    p_data[XDS_MAX_DATA_SIZE];
     int        i_sum;
 } xds_packet_t;
@@ -300,14 +300,14 @@ static int Open(vlc_object_t *p_this)
 
     /* peek at the first 12 bytes. */
     /* for TY streams, they're always the same */
-    if( stream_Peek( p_demux->s, &p_peek, 12 ) < 12 )
+    if( vlc_stream_Peek( p_demux->s, &p_peek, 12 ) < 12 )
         return VLC_EGENERIC;
 
     if ( U32_AT(p_peek) != TIVO_PES_FILEID ||
          U32_AT(&p_peek[4]) != 0x02 ||
          U32_AT(&p_peek[8]) != CHUNK_SIZE )
     {
-        if( !p_demux->b_force &&
+        if( !p_demux->obj.force &&
             !demux_IsPathExtension( p_demux, ".ty" ) &&
             !demux_IsPathExtension( p_demux, ".ty+" ) )
             return VLC_EGENERIC;
@@ -345,7 +345,7 @@ static int Open(vlc_object_t *p_this)
     p_sys->l_ac3_pkt_size = 0;
 
     /* see if this stream is seekable */
-    stream_Control( p_demux->s, STREAM_CAN_SEEK, &p_sys->b_seekable );
+    vlc_stream_Control( p_demux->s, STREAM_CAN_SEEK, &p_sys->b_seekable );
 
     if (probe_stream(p_demux) != VLC_SUCCESS) {
         //TyClose(p_demux);
@@ -402,11 +402,11 @@ static int Demux( demux_t *p_demux )
 
     /*
      * what we do (1 record now.. maybe more later):
-    * - use stream_Read() to read the chunk header & record headers
+    * - use vlc_stream_Read() to read the chunk header & record headers
     * - discard entire chunk if it is a PART header chunk
     * - parse all the headers into record header array
     * - keep a pointer of which record we're on
-    * - use stream_Block() to fetch each record
+    * - use vlc_stream_Block() to fetch each record
     * - parse out PTS from PES headers
     * - set PTS for data packets
     * - pass the data on to the proper codec via es_out_Send()
@@ -441,7 +441,7 @@ static int Demux( demux_t *p_demux )
         }
 
         /* read in this record's payload */
-        if( !( p_block_in = stream_Block( p_demux->s, l_rec_size ) ) )
+        if( !( p_block_in = vlc_stream_Block( p_demux->s, l_rec_size ) ) )
             return 0;
 
         /* set these as 'unknown' for now */
@@ -456,38 +456,29 @@ static int Demux( demux_t *p_demux )
                 p_rec->rec_type, p_rec->ex1, p_rec->ex2);
     }*/
 
-    if( p_rec->rec_type == 0xe0 )
+    switch( p_rec->rec_type )
     {
-        /* Video */
-        DemuxRecVideo( p_demux, p_rec, p_block_in );
-    }
-    else if ( p_rec->rec_type == 0xc0 )
-    {
-        /* Audio */
-        DemuxRecAudio( p_demux, p_rec, p_block_in );
-    }
-    else if( p_rec->rec_type == 0x01 || p_rec->rec_type == 0x02 )
-    {
-        /* Closed Captions/XDS */
-        DemuxRecCc( p_demux, p_rec, p_block_in );
-    }
-    else if ( p_rec->rec_type == 0x03 )
-    {
-        /* Tivo data services (e.g. "thumbs-up to record!")  useless for us */
-        if( p_block_in )
-            block_Release(p_block_in);
-    }
-    else if ( p_rec->rec_type == 0x05 )
-    {
-        /* Unknown, but seen regularly */
-        if( p_block_in )
-            block_Release(p_block_in);
-    }
-    else
-    {
-        msg_Dbg(p_demux, "Invalid record type 0x%02x", p_rec->rec_type );
-        if( p_block_in )
-            block_Release(p_block_in);
+        case 0xe0: /* video */
+            DemuxRecVideo( p_demux, p_rec, p_block_in );
+            break;
+
+        case 0xc0: /* audio */
+            DemuxRecAudio( p_demux, p_rec, p_block_in );
+            break;
+
+        case 0x01:
+        case 0x02:
+            /* closed captions/XDS */
+            DemuxRecCc( p_demux, p_rec, p_block_in );
+            break;
+
+        default:
+            msg_Dbg(p_demux, "Invalid record type 0x%02x", p_rec->rec_type );
+
+        case 0x03: /* tivo data services */
+        case 0x05: /* unknown, but seen regularly */
+            if( p_block_in )
+                block_Release( p_block_in );
     }
 
     /* */
@@ -513,8 +504,8 @@ static int Control(demux_t *p_demux, int i_query, va_list args)
         /* arg is 0.0 - 1.0 percent of overall file position */
         if( ( i64 = p_sys->i_stream_size ) > 0 )
         {
-            pf = (double*) va_arg( args, double* );
-            *pf = ((double)1.0) * stream_Tell( p_demux->s ) / (double) i64;
+            pf = va_arg( args, double* );
+            *pf = ((double)1.0) * vlc_stream_Tell( p_demux->s ) / (double) i64;
             return VLC_SUCCESS;
         }
         return VLC_EGENERIC;
@@ -528,7 +519,7 @@ static int Control(demux_t *p_demux, int i_query, va_list args)
         return VLC_EGENERIC;
     case DEMUX_GET_TIME:
         /* return TiVo timestamp */
-        p_i64 = (int64_t *) va_arg(args, int64_t *);
+        p_i64 = va_arg(args, int64_t *);
         //*p_i64 = p_sys->lastAudioPTS - p_sys->firstAudioPTS;
         //*p_i64 = (p_sys->l_last_ty_pts / 1000) + (p_sys->lastAudioPTS -
         //    p_sys->l_last_ty_pts_sync);
@@ -536,11 +527,11 @@ static int Control(demux_t *p_demux, int i_query, va_list args)
         return VLC_SUCCESS;
     case DEMUX_GET_LENGTH:    /* length of program in microseconds, 0 if unk */
         /* size / bitrate */
-        p_i64 = (int64_t *) va_arg(args, int64_t *);
+        p_i64 = va_arg(args, int64_t *);
         *p_i64 = 0;
         return VLC_SUCCESS;
     case DEMUX_SET_TIME:      /* arg is time in microsecs */
-        i64 = (int64_t) va_arg( args, int64_t );
+        i64 = va_arg( args, int64_t );
         return ty_stream_seek_time(p_demux, i64 * 1000);
     case DEMUX_GET_FPS:
     default:
@@ -768,14 +759,12 @@ static int DemuxRecVideo( demux_t *p_demux, ty_rec_hdr_t *rec_hdr, block_t *p_bl
     }
 
     /* Register the CC decoders when needed */
-    for( i = 0; i < 4; i++ )
+    uint64_t i_chans = p_sys->cc.i_608channels;
+    for( i = 0; i_chans > 0; i++, i_chans >>= 1 )
     {
-        static const vlc_fourcc_t fcc[4] = {
-            VLC_FOURCC('c', 'c', '1', ' '),
-            VLC_FOURCC('c', 'c', '2', ' '),
-            VLC_FOURCC('c', 'c', '3', ' '),
-            VLC_FOURCC('c', 'c', '4', ' ')
-        };
+        if( (i_chans & 1) == 0 || p_sys->p_cc[i] )
+            continue;
+
         static const char *ppsz_description[4] = {
             N_("Closed captions 1"),
             N_("Closed captions 2"),
@@ -785,10 +774,9 @@ static int DemuxRecVideo( demux_t *p_demux, ty_rec_hdr_t *rec_hdr, block_t *p_bl
 
         es_format_t fmt;
 
-        if( !p_sys->cc.pb_present[i] || p_sys->p_cc[i] )
-            continue;
 
-        es_format_Init( &fmt, SPU_ES, fcc[i] );
+        es_format_Init( &fmt, SPU_ES, VLC_CODEC_CEA608 );
+        fmt.subs.cc.i_channel = i;
         fmt.psz_description = strdup( vlc_gettext(ppsz_description[i]) );
         fmt.i_group = TY_ES_GROUP;
         p_sys->p_cc[i] = es_out_Add( p_demux->out, &fmt );
@@ -1042,7 +1030,7 @@ static int DemuxRecCc( demux_t *p_demux, ty_rec_hdr_t *rec_hdr, block_t *p_block
     if( p_sys->cc.i_data + 3 > CC_MAX_DATA_SIZE )
         return 0;
 
-    cc_AppendData( &p_sys->cc, i_field, rec_hdr->ex );
+    cc_AppendData( &p_sys->cc, CC_PKT_BYTE0(i_field), rec_hdr->ex );
     return 0;
 }
 
@@ -1063,7 +1051,7 @@ static int ty_stream_seek_pct(demux_t *p_demux, double seek_pct)
     p_sys->i_cur_chunk = seek_pos / CHUNK_SIZE;
 
     /* try to read the part header (master chunk) if it's there */
-    if ( stream_Seek( p_demux->s, i_cur_part * TIVO_PART_LENGTH ))
+    if ( vlc_stream_Seek( p_demux->s, i_cur_part * TIVO_PART_LENGTH ))
     {
         /* can't seek stream */
         return VLC_EGENERIC;
@@ -1071,7 +1059,7 @@ static int ty_stream_seek_pct(demux_t *p_demux, double seek_pct)
     parse_master(p_demux);
 
     /* now for the actual chunk */
-    if ( stream_Seek( p_demux->s, p_sys->i_cur_chunk * CHUNK_SIZE))
+    if ( vlc_stream_Seek( p_demux->s, p_sys->i_cur_chunk * CHUNK_SIZE))
     {
         /* can't seek stream */
         return VLC_EGENERIC;
@@ -1092,7 +1080,7 @@ static int ty_stream_seek_pct(demux_t *p_demux, double seek_pct)
     l_skip_amt = 0;
     for ( int i=0; i<p_sys->i_cur_rec; i++)
         l_skip_amt += p_sys->rec_hdrs[i].l_rec_size;
-    stream_Seek(p_demux->s, ((p_sys->i_cur_chunk-1) * CHUNK_SIZE) +
+    vlc_stream_Seek(p_demux->s, ((p_sys->i_cur_chunk-1) * CHUNK_SIZE) +
                  (p_sys->i_num_recs * 16) + l_skip_amt + 4);
 
     /* to hell with syncing any audio or video, just start reading records... :) */
@@ -1130,10 +1118,10 @@ static void XdsExit( xds_t *h )
     free( h->meta.future.psz_name );
     free( h->meta.future.psz_rating );
 }
-static void XdsStringUtf8( char dst[2*32+1], const uint8_t *p_src, int i_src )
+static void XdsStringUtf8( char dst[2*32+1], const uint8_t *p_src, size_t i_src )
 {
-    int i_dst = 0;
-    for( int i = 0; i < i_src; i++ )
+    size_t i_dst = 0;
+    for( size_t i = 0; i < i_src; i++ )
     {
         switch( p_src[i] )
         {
@@ -1430,7 +1418,6 @@ static void DemuxDecodeXds( demux_t *p_demux, uint8_t d1, uint8_t d2 )
     {
         xds_meta_t *m = &p_sys->xds.meta;
         vlc_meta_t *p_meta;
-        vlc_epg_t *p_epg;
 
         /* Channel meta data */
         p_meta = vlc_meta_New();
@@ -1444,20 +1431,32 @@ static void DemuxDecodeXds( demux_t *p_demux, uint8_t d1, uint8_t d2 )
         vlc_meta_Delete( p_meta );
 
         /* Event meta data (current/future) */
-        p_epg = vlc_epg_New( NULL );
         if( m->current.psz_name )
         {
-            vlc_epg_AddEvent( p_epg, 0, 0, m->current.psz_name, NULL, NULL, 0 );
-            //if( m->current.psz_rating )
-            //  TODO but VLC cannot yet handle rating per epg event
-            vlc_epg_SetCurrent( p_epg, 0 );
+            vlc_epg_t *p_epg = vlc_epg_New( TY_ES_GROUP, TY_ES_GROUP );
+            if ( p_epg )
+            {
+                vlc_epg_event_t *p_evt = vlc_epg_event_New( 0, 0, 0 );
+                if ( p_evt )
+                {
+                    if( m->current.psz_name )
+                        p_evt->psz_name = strdup( m->current.psz_name );
+                    if( !vlc_epg_AddEvent( p_epg, p_evt ) )
+                        vlc_epg_event_Delete( p_evt );
+                }
+                //if( m->current.psz_rating )
+                //  TODO but VLC cannot yet handle rating per epg event
+                vlc_epg_SetCurrent( p_epg, 0 );
+
+                if( m->future.psz_name )
+                {
+                }
+                if( p_epg->i_event > 0 )
+                    es_out_Control( p_demux->out, ES_OUT_SET_GROUP_EPG,
+                                    TY_ES_GROUP, p_epg );
+                vlc_epg_Delete( p_epg );
+            }
         }
-        if( m->future.psz_name )
-        {
-        }
-        if( p_epg->i_event > 0 )
-            es_out_Control( p_demux->out, ES_OUT_SET_GROUP_EPG, TY_ES_GROUP, p_epg );
-        vlc_epg_Delete( p_epg );
     }
     p_demux->p_sys->xds.b_meta_changed = false;
 }
@@ -1471,7 +1470,7 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
     unsigned i_seq_entry = 0;
     unsigned i;
     int i_skip_cnt;
-    int64_t l_cur_pos = stream_Tell(p_demux->s);
+    int64_t l_cur_pos = vlc_stream_Tell(p_demux->s);
     unsigned i_cur_part = l_cur_pos / TIVO_PART_LENGTH;
     uint64_t l_seek_secs = l_seek_time / 1000000000;
     uint64_t l_fwd_stamp = 1;
@@ -1489,11 +1488,11 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
         msg_Dbg(p_demux, "skipping to prior segment.");
         /* load previous part */
         if (i_cur_part == 0) {
-            stream_Seek(p_demux->s, l_cur_pos);
+            vlc_stream_Seek(p_demux->s, l_cur_pos);
             msg_Err(p_demux, "Attempt to seek past BOF");
             return VLC_EGENERIC;
         }
-        stream_Seek(p_demux->s, (i_cur_part - 1) * TIVO_PART_LENGTH);
+        vlc_stream_Seek(p_demux->s, (i_cur_part - 1) * TIVO_PART_LENGTH);
         i_cur_part--;
         parse_master(p_demux);
     }
@@ -1503,11 +1502,11 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
         /* load next part */
         if ((i_cur_part + 1) * TIVO_PART_LENGTH > p_sys->i_stream_size) {
             /* error; restore previous file position */
-            stream_Seek(p_demux->s, l_cur_pos);
+            vlc_stream_Seek(p_demux->s, l_cur_pos);
             msg_Err(p_demux, "seek error");
             return VLC_EGENERIC;
         }
-        stream_Seek(p_demux->s, (i_cur_part + 1) * TIVO_PART_LENGTH);
+        vlc_stream_Seek(p_demux->s, (i_cur_part + 1) * TIVO_PART_LENGTH);
         i_cur_part++;
         parse_master(p_demux);
     }
@@ -1533,18 +1532,18 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
     if (i == p_sys->i_seq_table_size) {
         if ((i_cur_part + 1) * TIVO_PART_LENGTH > p_sys->i_stream_size) {
             /* error; restore previous file position */
-            stream_Seek(p_demux->s, l_cur_pos);
+            vlc_stream_Seek(p_demux->s, l_cur_pos);
             msg_Err(p_demux, "seek error");
             return VLC_EGENERIC;
         }
-        stream_Seek(p_demux->s, (i_cur_part + 1) * TIVO_PART_LENGTH);
+        vlc_stream_Seek(p_demux->s, (i_cur_part + 1) * TIVO_PART_LENGTH);
         i_cur_part++;
         parse_master(p_demux);
         i_seq_entry = 0;
     }
 
     /* determine which chunk has our seek_time */
-    for (unsigned i=0; i<p_sys->i_bits_per_seq_entry; i++) {
+    for (i=0; i<p_sys->i_bits_per_seq_entry; i++) {
         uint64_t l_chunk_nr = i_seq_entry * p_sys->i_bits_per_seq_entry + i;
         uint64_t l_chunk_offset = (l_chunk_nr + 1) * CHUNK_SIZE;
         msg_Dbg(p_demux, "testing part %d chunk %"PRIu64" mask 0x%02X bit %d",
@@ -1554,7 +1553,7 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
             /* check this chunk's SEQ header timestamp */
             msg_Dbg(p_demux, "has SEQ. seeking to chunk at 0x%"PRIu64,
                 (i_cur_part * TIVO_PART_LENGTH) + l_chunk_offset);
-            stream_Seek(p_demux->s, (i_cur_part * TIVO_PART_LENGTH) +
+            vlc_stream_Seek(p_demux->s, (i_cur_part * TIVO_PART_LENGTH) +
                 l_chunk_offset);
             // TODO: we don't have to parse the full header set;
             // just test the seq_rec entry for its timestamp
@@ -1564,7 +1563,7 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
             if (p_sys->i_seq_rec < 0 || p_sys->i_seq_rec > p_sys->i_num_recs) {
                 msg_Err(p_demux, "no SEQ hdr in chunk; table had one.");
                 /* Seek to beginning of original chunk & reload it */
-                stream_Seek(p_demux->s, (l_cur_pos / CHUNK_SIZE) * CHUNK_SIZE);
+                vlc_stream_Seek(p_demux->s, (l_cur_pos / CHUNK_SIZE) * CHUNK_SIZE);
                 p_sys->i_stuff_cnt = 0;
                 get_chunk_header(p_demux);
                 return VLC_EGENERIC;
@@ -1597,7 +1596,7 @@ static int ty_stream_seek_time(demux_t *p_demux, uint64_t l_seek_time)
     i_skip_cnt = 0;
     for (int j=0; j<p_sys->i_seq_rec; j++)
         i_skip_cnt += p_sys->rec_hdrs[j].l_rec_size;
-    stream_Read(p_demux->s, NULL, i_skip_cnt);
+    vlc_stream_Read(p_demux->s, NULL, i_skip_cnt);
     p_sys->i_cur_rec = p_sys->i_seq_rec;
     //p_sys->l_last_ty_pts = p_sys->rec_hdrs[p_sys->i_seq_rec].l_ty_pts;
     //p_sys->l_last_ty_pts_sync = p_sys->lastAudioPTS;
@@ -1613,8 +1612,7 @@ static void parse_master(demux_t *p_demux)
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     uint8_t mst_buf[32];
-    uint32_t i, i_map_size;
-    int64_t i_save_pos = stream_Tell(p_demux->s);
+    int64_t i_save_pos = vlc_stream_Tell(p_demux->s);
     int64_t i_pts_secs;
 
     /* Note that the entries in the SEQ table in the stream may have
@@ -1627,12 +1625,20 @@ static void parse_master(demux_t *p_demux)
     free(p_sys->seq_table);
 
     /* parse header info */
-    stream_Read(p_demux->s, mst_buf, 32);
-    i_map_size = U32_AT(&mst_buf[20]);  /* size of bitmask, in bytes */
+    vlc_stream_Read(p_demux->s, mst_buf, 32);
+
+    uint32_t i_map_size = U32_AT(&mst_buf[20]);  /* size of bitmask, in bytes */
+    uint32_t i = U32_AT(&mst_buf[28]);   /* size of SEQ table, in bytes */
+
     p_sys->i_bits_per_seq_entry = i_map_size * 8;
-    i = U32_AT(&mst_buf[28]);   /* size of SEQ table, in bytes */
     p_sys->i_seq_table_size = i / (8 + i_map_size);
 
+    if(p_sys->i_seq_table_size == 0)
+    {
+        p_sys->seq_table = NULL;
+        return;
+    }
+
     /* parse all the entries */
     p_sys->seq_table = calloc(p_sys->i_seq_table_size, sizeof(ty_seq_table_t));
     if (p_sys->seq_table == NULL)
@@ -1640,15 +1646,15 @@ static void parse_master(demux_t *p_demux)
         p_sys->i_seq_table_size = 0;
         return;
     }
-    for (unsigned i=0; i<p_sys->i_seq_table_size; i++) {
-        stream_Read(p_demux->s, mst_buf, 8);
-        p_sys->seq_table[i].l_timestamp = U64_AT(&mst_buf[0]);
+    for (unsigned j=0; j<p_sys->i_seq_table_size; j++) {
+        vlc_stream_Read(p_demux->s, mst_buf, 8);
+        p_sys->seq_table[j].l_timestamp = U64_AT(&mst_buf[0]);
         if (i_map_size > 8) {
             msg_Err(p_demux, "Unsupported SEQ bitmap size in master chunk");
-            stream_Read(p_demux->s, NULL, i_map_size);
+            vlc_stream_Read(p_demux->s, NULL, i_map_size);
         } else {
-            stream_Read(p_demux->s, mst_buf + 8, i_map_size);
-            memcpy(p_sys->seq_table[i].chunk_bitmask, &mst_buf[8], i_map_size);
+            vlc_stream_Read(p_demux->s, mst_buf + 8, i_map_size);
+            memcpy(p_sys->seq_table[j].chunk_bitmask, &mst_buf[8], i_map_size);
         }
     }
 
@@ -1668,7 +1674,7 @@ static void parse_master(demux_t *p_demux)
              i_pts_secs / 3600, (i_pts_secs / 60) % 60, i_pts_secs % 60 );
 
     /* seek past this chunk */
-    stream_Seek(p_demux->s, i_save_pos + CHUNK_SIZE);
+    vlc_stream_Seek(p_demux->s, i_save_pos + CHUNK_SIZE);
 }
 
 
@@ -1686,7 +1692,7 @@ static int probe_stream(demux_t *p_demux)
     bool b_probe_error = false;
 
     /* we need CHUNK_PEEK_COUNT chunks of data, first one might be a Part header, so ... */
-    if (stream_Peek( p_demux->s, &p_buf, CHUNK_PEEK_COUNT * CHUNK_SIZE ) <
+    if (vlc_stream_Peek( p_demux->s, &p_buf, CHUNK_PEEK_COUNT * CHUNK_SIZE ) <
             CHUNK_PEEK_COUNT * CHUNK_SIZE) {
         msg_Err(p_demux, "Can't peek %d chunks", CHUNK_PEEK_COUNT);
         /* TODO: if seekable, then loop reading chunks into a temp buffer */
@@ -1845,12 +1851,12 @@ static int get_chunk_header(demux_t *p_demux)
 
     /* if we have left-over filler space from the last chunk, get that */
     if (p_sys->i_stuff_cnt > 0) {
-        stream_Read( p_demux->s, NULL, p_sys->i_stuff_cnt);
+        vlc_stream_Read( p_demux->s, NULL, p_sys->i_stuff_cnt);
         p_sys->i_stuff_cnt = 0;
     }
 
     /* read the TY packet header */
-    i_readSize = stream_Peek( p_demux->s, &p_peek, 4 );
+    i_readSize = vlc_stream_Peek( p_demux->s, &p_peek, 4 );
     p_sys->i_cur_chunk++;
 
     if ( (i_readSize < 4) || ( U32_AT(&p_peek[ 0 ] ) == 0 ))
@@ -1894,11 +1900,11 @@ static int get_chunk_header(demux_t *p_demux)
     p_sys->rec_hdrs = NULL;
 
     /* skip past the 4 bytes we "peeked" earlier */
-    stream_Read( p_demux->s, NULL, 4 );
+    vlc_stream_Read( p_demux->s, NULL, 4 );
 
     /* read the record headers into a temp buffer */
     p_hdr_buf = xmalloc(i_num_recs * 16);
-    if (stream_Read(p_demux->s, p_hdr_buf, i_num_recs * 16) < i_num_recs * 16) {
+    if (vlc_stream_Read(p_demux->s, p_hdr_buf, i_num_recs * 16) < i_num_recs * 16) {
         free( p_hdr_buf );
         p_sys->eof = true;
         return 0;
