@@ -2,7 +2,7 @@
  * profile_selector.cpp : A small profile selector and editor
  ****************************************************************************
  * Copyright (C) 2009 the VideoLAN team
- * $Id: cfb18825bde1b69573714d6126635a073c064b2c $
+ * $Id: 6fae3a2eee6051bee060ea6dbed9a08aa3d1fe03 $
  *
  * Authors: Jean-Baptiste Kempf <jb@videolan.org>
  *
@@ -37,6 +37,7 @@
 #include <QSpinBox>
 #include <QUrl>
 #include <QListWidgetItem>
+#include <QFontMetrics>
 
 #include <assert.h>
 #include <vlc_modules.h>
@@ -56,17 +57,17 @@ VLCProfileSelector::VLCProfileSelector( QWidget *_parent ): QWidget( _parent )
     layout->addWidget( profileBox );
 
     QToolButton *editButton = new QToolButton( this );
-    editButton->setIcon( QIcon( ":/menu/preferences" ) );
+    editButton->setIcon( QIcon( ":/menu/preferences.svg" ) );
     editButton->setToolTip( qtr( "Edit selected profile" ) );
     layout->addWidget( editButton );
 
     QToolButton *deleteButton = new QToolButton( this );
-    deleteButton->setIcon( QIcon( ":/toolbar/clear" ) );
+    deleteButton->setIcon( QIcon( ":/toolbar/clear.svg" ) );
     deleteButton->setToolTip( qtr( "Delete selected profile" ) );
     layout->addWidget( deleteButton );
 
     QToolButton *newButton = new QToolButton( this );
-    newButton->setIcon( QIcon( ":/new" ) );
+    newButton->setIcon( QIcon( ":/new.svg" ) );
     newButton->setToolTip( qtr( "Create a new profile" ) );
     layout->addWidget(newButton);
 
@@ -342,12 +343,18 @@ void VLCProfileSelector::updateOptions( int i )
     HASHPICK( "subtitles", "enable" );
     if( !value.isEmpty() )
     {
-        HASHPICK( "subtitles", "codec" );
-        smrl.option( "scodec", value );
-
         HASHPICK( "subtitles", "overlay" );
-        if ( !value.isEmpty() )
+        if ( value.isEmpty() )
+        {
+            HASHPICK( "subtitles", "codec" );
+            smrl.option( "scodec", value );
+        }
+        else
+        {
             smrl.option( "soverlay" );
+        }
+    } else {
+        smrl.option( "scodec", "none" );
     }
     smrl.end();
 #undef HASHPICK
@@ -450,6 +457,8 @@ VLCProfileEditor::VLCProfileEditor( const QString& qs_name, const QString& value
              this, activatePanels() );
     CONNECT( ui.valueholder_audio_copy, stateChanged( int ),
              this, activatePanels() );
+    CONNECT( ui.valueholder_subtitles_overlay, stateChanged( int ),
+             this, activatePanels() );
     CONNECT( ui.valueholder_vcodec_bitrate, editingFinished( ),
              this, fixBirateState() );
     CONNECT( ui.valueholder_vcodec_qp, editingFinished( ),
@@ -495,7 +504,7 @@ inline void VLCProfileEditor::registerFilters()
         QListWidget *listWidget = NULL;
         QListWidgetItem *item;
 
-        if ( capability == "video filter2" )
+        if ( capability == "video filter" )
             listWidget = ui.valueholder_video_filters;
         else if ( capability == "audio filter" )
             listWidget = ui.valueholder_audio_filters;
@@ -618,16 +627,20 @@ void VLCProfileEditor::muxSelected()
     SETYESNOSTATE( capsubs, "capsubs" );
     SETYESNOSTATE( capstream, "capstream" );
     SETYESNOSTATE( capchaps, "capchaps" );
+
+    int textsize = QFontMetrics(ui.muxerwarning->font()).ascent();
     if( current->property("module").toString() == "avformat" )
         ui.muxerwarning->setText(
-                    QString( "<img src=\":/menu/info\"/> %1" )
+                    QString( "<img src=\":/menu/info.svg\" width=%2 height=%2/> %1" )
                     .arg( qtr( "This muxer is not provided directly by VLC: It could be missing." ) )
+                    .arg(textsize)
                     );
     else if ( !caps["muxers"].contains( current->property("module").toString() ) &&
               !caps["muxers"].contains( "mux_" + current->property("module").toString() ) )
         ui.muxerwarning->setText(
-                    QString( "<img src=\":/menu/quit\"/> %1" )
+                    QString( "<img src=\":/toobar/clear.svg\" width=%2 height=%2/> %1" )
                     .arg( qtr( "This muxer is missing. Using this profile will fail" ) )
+                    .arg(textsize)
                     );
     else
         ui.muxerwarning->setText("");
@@ -876,6 +889,7 @@ void VLCProfileEditor::activatePanels()
 {
     ui.transcodevideo->setEnabled( ! ui.valueholder_video_copy->isChecked() );
     ui.transcodeaudio->setEnabled( ! ui.valueholder_audio_copy->isChecked() );
+    ui.valueholder_subtitles_codec->setEnabled( ! ui.valueholder_subtitles_overlay->isChecked() );
 }
 
 void VLCProfileEditor::fixBirateState()
