@@ -187,26 +187,36 @@ local options = {
 }
 
 local languages = {
+  {'abk', 'Abkhazian'},
+  {'afr', 'Afrikaans'},
   {'alb', 'Albanian'},
   {'ara', 'Arabic'},
+  {'arg', 'Aragonese'},
   {'arm', 'Armenian'},
+  {'ast', 'Asturian'},
+  {'aze', 'Azerbaijani'},
   {'baq', 'Basque'},
+  {'bel', 'Belarusian'},
   {'ben', 'Bengali'},
   {'bos', 'Bosnian'},
   {'bre', 'Breton'},
   {'bul', 'Bulgarian'},
   {'bur', 'Burmese'},
   {'cat', 'Catalan'},
-  {'chi', 'Chinese'},
+  {'chi', 'Chinese (simplified)'},
+  {'zht', 'Chinese (traditional)'},
   {'hrv', 'Croatian'},
   {'cze', 'Czech'},
   {'dan', 'Danish'},
+  {'prs', 'Dari'},
   {'dut', 'Dutch'},
   {'eng', 'English'},
   {'epo', 'Esperanto'},
   {'est', 'Estonian'},
+  {'ext', 'Extremaduran'},
   {'fin', 'Finnish'},
   {'fre', 'French'},
+  {'gla', 'Gaelic'},
   {'glg', 'Galician'},
   {'geo', 'Georgian'},
   {'ger', 'German'},
@@ -215,28 +225,42 @@ local languages = {
   {'hin', 'Hindi'},
   {'hun', 'Hungarian'},
   {'ice', 'Icelandic'},
+  {'ibo', 'Igbo'},
   {'ind', 'Indonesian'},
+  {'gle', 'Irish'},
   {'ita', 'Italian'},
   {'jpn', 'Japanese'},
+  {'kan', 'Kannada'},
   {'kaz', 'Kazakh'},
   {'khm', 'Khmer'},
   {'kor', 'Korean'},
+  {'kur', 'Kurdish'},
   {'lav', 'Latvian'},
   {'lit', 'Lithuanian'},
   {'ltz', 'Luxembourgish'},
   {'mac', 'Macedonian'},
   {'may', 'Malay'},
   {'mal', 'Malayalam'},
+  {'mni', 'Manipuri'},
+  {'mar', 'Marathi'},
   {'mon', 'Mongolian'},
+  {'mne', 'Montenegrin'},
+  {'nav', 'Navajo'},
+  {'nep', 'Nepali'},
+  {'sme', 'Northern Sami'},
   {'nor', 'Norwegian'},
   {'oci', 'Occitan'},
+  {'ori', 'Odia'},
   {'per', 'Persian'},
   {'pol', 'Polish'},
   {'por', 'Portuguese'},
   {'pob', 'Brazilian Portuguese'},
+  {'pus', 'Pushto'},
   {'rum', 'Romanian'},
   {'rus', 'Russian'},
+  {'sat', 'Santali'},
   {'scc', 'Serbian'},
+  {'snd', 'Sindhi'},
   {'sin', 'Sinhalese'},
   {'slo', 'Slovak'},
   {'slv', 'Slovenian'},
@@ -245,12 +269,17 @@ local languages = {
   {'swe', 'Swedish'},
   {'syr', 'Syriac'},
   {'tgl', 'Tagalog'},
+  {'tam', 'Tamil'},
+  {'tat', 'Tatar'},
   {'tel', 'Telugu'},
   {'tha', 'Thai'},
+  {'tok', 'Toki Pona'},
   {'tur', 'Turkish'},
+  {'tuk', 'Turkmen'},
   {'ukr', 'Ukrainian'},
   {'urd', 'Urdu'},
-  {'vie', 'Vietnamese'}
+  {'vie', 'Vietnamese'},
+  {'wel', 'Welsh'}
 }
 
 -- Languages code conversion table: iso-639-1 to iso-639-3
