@@ -2,7 +2,7 @@
  * decoder.c: Functions for the management of decoders
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: 775f256313cd2a61eb9187bc72b5af419c808b70 $
+ * $Id: df061f98cacb365330f460c3afed9d599381a375 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -405,6 +405,8 @@ static int aout_update_format( decoder_t *p_dec )
 
         p_dec->fmt_out.audio.i_bytes_per_frame =
             p_owner->fmt.audio.i_bytes_per_frame;
+        p_dec->fmt_out.audio.i_bitspersample =
+            p_owner->fmt.audio.i_bitspersample;
         p_dec->fmt_out.audio.i_frame_length =
             p_owner->fmt.audio.i_frame_length;
     }
@@ -510,6 +512,9 @@ static int vout_update_format( decoder_t *p_dec )
         case VLC_CODEC_DIRAC: /* FIXME valid ? */
             dpb_size = 18;
             break;
+        case VLC_CODEC_AV1:
+            dpb_size = 8; /* NUM_REF_FRAMES from the AV1 spec */
+            break;
         case VLC_CODEC_VP5:
         case VLC_CODEC_VP6:
         case VLC_CODEC_VP6F:
