@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2008 VLC authors and VideoLAN
  * Copyright (C) 2008 Laurent Aimar
- * $Id: 4d63fbf9c45d3ce239642aee0e45a643171ebf3c $
+ * $Id: 400bcb8b61b5173c08f9eb19e4dbade57533ce8d $
  *
  * Authors: Laurent Aimar < fenrir _AT_ videolan _DOT_ org >
  *
@@ -109,6 +109,11 @@ struct vout_thread_sys_t
         int         position;
     } title;
 
+    struct {
+        bool        is_interlaced;
+        mtime_t     date;
+    } interlacing;
+
     /* */
     bool            is_late_dropped;
 
@@ -119,6 +124,7 @@ struct vout_thread_sys_t
         video_format_t  format;
         struct filter_chain_t *chain_static;
         struct filter_chain_t *chain_interactive;
+        bool            has_deint;
     } filter;
 
     /* */
@@ -145,6 +151,7 @@ void vout_ControlChangeFilters(vout_thread_t *, const char *);
 void vout_ControlChangeSubSources(vout_thread_t *, const char *);
 void vout_ControlChangeSubFilters(vout_thread_t *, const char *);
 void vout_ControlChangeSubMargin(vout_thread_t *, int);
+void vout_ControlChangeViewpoint( vout_thread_t *, const vlc_viewpoint_t *);
 
 /* */
 void vout_IntfInit( vout_thread_t * );
